"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Drawing.py,v 1.1.2.1 2003/03/11 19:37:13 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
import Parameters as wx


class DC(Object):
    """"""

    def BeginDrawing(self):
        """"""
        pass

    def Blit(self):
        """"""
        pass

    def CalcBoundingBox(self):
        """"""
        pass

    def CanDrawBitmap(self):
        """"""
        pass

    def CanGetTextExtent(self):
        """"""
        pass

    def Clear(self):
        """"""
        pass

    def CrossHair(self):
        """"""
        pass

    def DestroyClippingRegion(self):
        """"""
        pass

    def DeviceToLogicalX(self):
        """"""
        pass

    def DeviceToLogicalXRel(self):
        """"""
        pass

    def DeviceToLogicalY(self):
        """"""
        pass

    def DeviceToLogicalYRel(self):
        """"""
        pass

    def DrawArc(self):
        """"""
        pass

    def DrawBitmap(self):
        """"""
        pass

    def DrawCircle(self):
        """"""
        pass

    def DrawEllipse(self):
        """"""
        pass

    def DrawEllipticArc(self):
        """"""
        pass

    def DrawIcon(self):
        """"""
        pass

    def DrawImageLabel(self):
        """"""
        pass

    def DrawLabel(self):
        """"""
        pass

    def DrawLine(self):
        """"""
        pass

    def DrawLineList(self):
        """"""
        pass

    def DrawLines(self):
        """"""
        pass

    def DrawPoint(self):
        """"""
        pass

    def DrawPointList(self):
        """"""
        pass

    def DrawPolygon(self):
        """"""
        pass

    def DrawRectangle(self):
        """"""
        pass

    def DrawRectangleRect(self):
        """"""
        pass

    def DrawRotatedText(self):
        """"""
        pass

    def DrawRoundedRectangle(self):
        """"""
        pass

    def DrawSpline(self):
        """"""
        pass

    def DrawText(self):
        """"""
        pass

    def EndDoc(self):
        """"""
        pass

    def EndDrawing(self):
        """"""
        pass

    def EndPage(self):
        """"""
        pass

    def FloodFill(self):
        """"""
        pass

    def GetBackground(self):
        """"""
        pass

    def GetBoundingBox(self):
        """"""
        pass

    def GetBrush(self):
        """"""
        pass

    def GetCharHeight(self):
        """"""
        pass

    def GetCharWidth(self):
        """"""
        pass

    def GetClippingBox(self):
        """"""
        pass

    def GetDepth(self):
        """"""
        pass

    def GetDeviceOrigin(self):
        """"""
        pass

    def GetFont(self):
        """"""
        pass

    def GetFullTextExtent(self):
        """"""
        pass

    def GetLogicalFunction(self):
        """"""
        pass

    def GetLogicalOrigin(self):
        """"""
        pass

    def GetLogicalScale(self):
        """"""
        pass

    def GetMapMode(self):
        """"""
        pass

    def GetMultiLineTextExtent(self):
        """"""
        pass

    def GetOptimization(self):
        """"""
        pass

    def GetPPI(self):
        """"""
        pass

    def GetPen(self):
        """"""
        pass

    def GetPixel(self):
        """"""
        pass

    def GetSize(self):
        """"""
        pass

    def GetSizeMM(self):
        """"""
        pass

    def GetSizeTuple(self):
        """"""
        pass

    def GetTextBackground(self):
        """"""
        pass

    def GetTextExtent(self):
        """"""
        pass

    def GetTextForeground(self):
        """"""
        pass

    def GetUserScale(self):
        """"""
        pass

    def LogicalToDeviceX(self):
        """"""
        pass

    def LogicalToDeviceXRel(self):
        """"""
        pass

    def LogicalToDeviceY(self):
        """"""
        pass

    def LogicalToDeviceYRel(self):
        """"""
        pass

    def MaxX(self):
        """"""
        pass

    def MaxY(self):
        """"""
        pass

    def MinX(self):
        """"""
        pass

    def MinY(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def ResetBoundingBox(self):
        """"""
        pass

    def SetAxisOrientation(self):
        """"""
        pass

    def SetBackground(self):
        """"""
        pass

    def SetBackgroundMode(self):
        """"""
        pass

    def SetBrush(self):
        """"""
        pass

    def SetClippingRect(self):
        """"""
        pass

    def SetClippingRegion(self):
        """"""
        pass

    def SetClippingRegionAsRegion(self):
        """"""
        pass

    def SetDeviceOrigin(self):
        """"""
        pass

    def SetFont(self):
        """"""
        pass

    def SetLogicalFunction(self):
        """"""
        pass

    def SetLogicalOrigin(self):
        """"""
        pass

    def SetLogicalScale(self):
        """"""
        pass

    def SetMapMode(self):
        """"""
        pass

    def SetOptimization(self):
        """"""
        pass

    def SetPalette(self):
        """"""
        pass

    def SetPen(self):
        """"""
        pass

    def SetTextBackground(self):
        """"""
        pass

    def SetTextForeground(self):
        """"""
        pass

    def SetUserScale(self):
        """"""
        pass

    def StartDoc(self):
        """"""
        pass

    def StartPage(self):
        """"""
        pass

    def _DrawLineList(self):
        """"""
        pass

    def _DrawPointList(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ClientDC(DC):
    """"""

    def __init__(self):
        """"""
        pass


class MemoryDC(DC):
    """"""

    def SelectObject(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class BufferedDC(MemoryDC):
    """"""

    def UnMask(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class BufferedPaintDC(BufferedDC):
    """"""

    def __init__(self):
        """"""
        pass


class PaintDC(DC):
    """"""

    def __init__(self):
        """"""
        pass


class PostScriptDC(DC):
    """"""

    def GetPrintData(self):
        """"""
        pass

    def SetPrintData(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ScreenDC(DC):
    """"""

    def EndDrawingOnTop(self):
        """"""
        pass

    def StartDrawingOnTop(self):
        """"""
        pass

    def StartDrawingOnTopWin(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class WindowDC(DC):
    """"""

    def __init__(self):
        """"""
        pass


class GDIObject(Object):
    """"""

    def GetVisible(self):
        """"""
        pass

    def IsNull(self):
        """"""
        pass

    def SetVisible(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Bitmap(GDIObject):
    """"""

    def CopyFromIcon(self):
        """"""
        pass

    def GetDepth(self):
        """"""
        pass

    def GetHeight(self):
        """"""
        pass

    def GetMask(self):
        """"""
        pass

    def GetPalette(self):
        """"""
        pass

    def GetSubBitmap(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def LoadFile(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def SaveFile(self):
        """"""
        pass

    def SetDepth(self):
        """"""
        pass

    def SetHeight(self):
        """"""
        pass

    def SetMask(self):
        """"""
        pass

    def SetWidth(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Brush(GDIObject):
    """"""

    def GetColour(self):
        """"""
        pass

    def GetStipple(self):
        """"""
        pass

    def GetStyle(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def SetColour(self):
        """"""
        pass

    def SetStipple(self):
        """"""
        pass

    def SetStyle(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class BrushList(Object):
    """"""

    def AddBrush(self):
        """"""
        pass

    def FindOrCreateBrush(self):
        """"""
        pass

    def GetCount(self):
        """"""
        pass

    def RemoveBrush(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Colour(Object):
    """"""

    def Blue(self):
        """"""
        pass

    def Get(self):
        """"""
        pass

    def Green(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def Red(self):
        """"""
        pass

    def Set(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __eq__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def __ne__(self):
        """"""
        pass

    def __nonzero__(self):
        """"""
        pass

    def __str__(self):
        """"""
        pass

    def asTuple(self):
        """"""
        pass


class Cursor(GDIObject):
    """"""

    def Ok(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Font(Object):
    """"""

    def GetEncoding(self):
        """"""
        pass

    def GetFaceName(self):
        """"""
        pass

    def GetFamily(self):
        """"""
        pass

    def GetFamilyString(self):
        """"""
        pass

    def GetNativeFontInfo(self):
        """"""
        pass

    def GetNativeFontInfoDesc(self):
        """"""
        pass

    def GetNativeFontInfoUserDesc(self):
        """"""
        pass

    def GetNoAntiAliasing(self):
        """"""
        pass

    def GetPointSize(self):
        """"""
        pass

    def GetStyle(self):
        """"""
        pass

    def GetStyleString(self):
        """"""
        pass

    def GetUnderlined(self):
        """"""
        pass

    def GetWeight(self):
        """"""
        pass

    def GetWeightString(self):
        """"""
        pass

    def IsFixedWidth(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def SetEncoding(self):
        """"""
        pass

    def SetFaceName(self):
        """"""
        pass

    def SetFamily(self):
        """"""
        pass

    def SetNativeFontInfo(self):
        """"""
        pass

    def SetNativeFontInfoUserDesc(self):
        """"""
        pass

    def SetNoAntiAliasing(self):
        """"""
        pass

    def SetPointSize(self):
        """"""
        pass

    def SetStyle(self):
        """"""
        pass

    def SetUnderlined(self):
        """"""
        pass

    def SetWeight(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass

    def wxFontFromNativeInfoString(self):
        """"""
        pass


class FontList(Object):
    """"""

    def AddFont(self):
        """"""
        pass

    def FindOrCreateFont(self):
        """"""
        pass

    def GetCount(self):
        """"""
        pass

    def RemoveFont(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Icon(GDIObject):
    """"""

    def CopyFromBitmap(self):
        """"""
        pass

    def GetDepth(self):
        """"""
        pass

    def GetHeight(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def LoadFile(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def SetDepth(self):
        """"""
        pass

    def SetHeight(self):
        """"""
        pass

    def SetWidth(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class IconBundle:
    """"""

    def AddIcon(self):
        """"""
        pass

    def AddIconFromFile(self):
        """"""
        pass

    def GetIcon(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Image(Object):
    """"""

    def ConvertToBitmap(self):
        """"""
        pass

    def ConvertToMono(self):
        """"""
        pass

    def ConvertToMonoBitmap(self):
        """"""
        pass

    def Copy(self):
        """"""
        pass

    def CountColours(self):
        """"""
        pass

    def Create(self):
        """"""
        pass

    def Destroy(self):
        """"""
        pass

    def FindFirstUnusedColour(self):
        """"""
        pass

    def GetBlue(self):
        """"""
        pass

    def GetData(self):
        """"""
        pass

    def GetDataBuffer(self):
        """"""
        pass

    def GetGreen(self):
        """"""
        pass

    def GetHeight(self):
        """"""
        pass

    def GetMaskBlue(self):
        """"""
        pass

    def GetMaskGreen(self):
        """"""
        pass

    def GetMaskRed(self):
        """"""
        pass

    def GetOption(self):
        """"""
        pass

    def GetOptionInt(self):
        """"""
        pass

    def GetRed(self):
        """"""
        pass

    def GetSubImage(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def HasMask(self):
        """"""
        pass

    def HasOption(self):
        """"""
        pass

    def LoadFile(self):
        """"""
        pass

    def LoadMimeFile(self):
        """"""
        pass

    def LoadMimeStream(self):
        """"""
        pass

    def LoadStream(self):
        """"""
        pass

    def Mirror(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def Paste(self):
        """"""
        pass

    def Replace(self):
        """"""
        pass

    def Rescale(self):
        """"""
        pass

    def Rotate(self):
        """"""
        pass

    def Rotate90(self):
        """"""
        pass

    def SaveFile(self):
        """"""
        pass

    def SaveMimeFile(self):
        """"""
        pass

    def Scale(self):
        """"""
        pass

    def SetData(self):
        """"""
        pass

    def SetMask(self):
        """"""
        pass

    def SetMaskColour(self):
        """"""
        pass

    def SetMaskFromImage(self):
        """"""
        pass

    def SetOption(self):
        """"""
        pass

    def SetOptionInt(self):
        """"""
        pass

    def SetRGB(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ImageList(Object):
    """"""

    def Add(self):
        """"""
        pass

    def AddIcon(self):
        """"""
        pass

    def AddWithColourMask(self):
        """"""
        pass

    def Draw(self):
        """"""
        pass

    def GetImageCount(self):
        """"""
        pass

    def GetSize(self):
        """"""
        pass

    def Remove(self):
        """"""
        pass

    def RemoveAll(self):
        """"""
        pass

    def Replace(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Mask(Object):
    """"""

    def Destroy(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Palette(GDIObject):
    """"""

    def GetPixel(self):
        """"""
        pass

    def GetRGB(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Pen(GDIObject):
    """"""

    def GetCap(self):
        """"""
        pass

    def GetColour(self):
        """"""
        pass

    def GetDashes(self):
        """"""
        pass

    def GetJoin(self):
        """"""
        pass

    def GetStyle(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def Ok(self):
        """"""
        pass

    def SetCap(self):
        """"""
        pass

    def SetColour(self):
        """"""
        pass

    def SetDashes(self):
        """"""
        pass

    def SetJoin(self):
        """"""
        pass

    def SetStyle(self):
        """"""
        pass

    def SetWidth(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class PenList(Object):
    """"""

    def AddPen(self):
        """"""
        pass

    def FindOrCreatePen(self):
        """"""
        pass

    def GetCount(self):
        """"""
        pass

    def RemovePen(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class PyPen(Pen):
    """"""

    def SetDashes(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class Region(GDIObject):
    """"""

    def Clear(self):
        """"""
        pass

    def Contains(self):
        """"""
        pass

    def ContainsPoint(self):
        """"""
        pass

    def ContainsRect(self):
        """"""
        pass

    def ContainsRectDim(self):
        """"""
        pass

    def GetBox(self):
        """"""
        pass

    def Intersect(self):
        """"""
        pass

    def IntersectRect(self):
        """"""
        pass

    def IntersectRegion(self):
        """"""
        pass

    def IsEmpty(self):
        """"""
        pass

    def Offset(self):
        """"""
        pass

    def Subtract(self):
        """"""
        pass

    def SubtractRect(self):
        """"""
        pass

    def SubtractRegion(self):
        """"""
        pass

    def Union(self):
        """"""
        pass

    def UnionRect(self):
        """"""
        pass

    def UnionRegion(self):
        """"""
        pass

    def Xor(self):
        """"""
        pass

    def XorRect(self):
        """"""
        pass

    def XorRegion(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class RegionIterator(Object):
    """"""

    def GetH(self):
        """"""
        pass

    def GetHeight(self):
        """"""
        pass

    def GetRect(self):
        """"""
        pass

    def GetW(self):
        """"""
        pass

    def GetWidth(self):
        """"""
        pass

    def GetX(self):
        """"""
        pass

    def GetY(self):
        """"""
        pass

    def HaveRects(self):
        """"""
        pass

    def Next(self):
        """"""
        pass

    def Reset(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass





#----------------------------------------------------------------------------
# Name:         BrowseImage.py
# Purpose:      Display and Select Image Files
#
# Author:       Lorne White
#
# Version:      1.0
# Date:         January 29, 2002
# Licence:      wxWindows license
#----------------------------------------------------------------------------

# 1.0 Release
# Create list of all available image file types
# View "All Image" File Types as default filter
# Sort the file list
# Use newer "re" function for patterns

#---------------------------------------------------------------------------

import os, sys
from wxPython.wx import *
dir_path = os.getcwd()

#---------------------------------------------------------------------------

def ConvertBMP(file_nm):
    if file_nm is None:
        return None

    fl_fld = os.path.splitext(file_nm)
    ext = fl_fld[1]
    ext = ext[1:].lower()
    if ext == 'bmp':
        image = wxImage(file_nm, wxBITMAP_TYPE_BMP)
    elif ext == 'gif':
        image = wxImage(file_nm, wxBITMAP_TYPE_GIF)
    elif ext == 'png':
        image = wxImage(file_nm, wxBITMAP_TYPE_PNG)
    elif ext == 'jpg':
        image = wxImage(file_nm, wxBITMAP_TYPE_JPEG)
    elif ext == 'pcx':
        image = wxImage(file_nm, wxBITMAP_TYPE_PCX)
    elif ext == 'tif':
        image = wxImage(file_nm, wxBITMAP_TYPE_TIF)
    elif ext == 'pnm':
        image = wxImage(file_nm, wxBITMAP_TYPE_PNM)
    else:
        image = wxImage(file_nm, wxBITMAP_TYPE_ANY)

    return image

def GetSize(file_nm):       # for scaling image values
    image = ConvertBMP(file_nm)
    bmp = image.ConvertToBitmap()
    size = bmp.GetWidth(), bmp.GetHeight()
    return size

class ImageView(wxWindow):
    def __init__(self, parent, id=-1, pos=wxDefaultPosition, size=wxDefaultSize):
        wxWindow.__init__(self, parent, id, pos, size)
        self.win = parent
        self.image = None
        self.back_color = 'WHITE'
        self.border_color = 'BLACK'

        self.image_sizex = size.width
        self.image_sizey = size.height
        self.image_posx = pos.x
        self.image_posy = pos.y
        EVT_PAINT(self, self.OnPaint)

        wxInitAllImageHandlers()

    def OnPaint(self, event):
        dc = wxPaintDC(self)
        self.DrawImage(dc)

    def DrawImage(self, dc):
        dc.BeginDrawing()
        self.DrawImage(dc)
        dc.EndDrawing()

    def SetValue(self, file_nm):    # display the selected file in the panel
        image = ConvertBMP(file_nm)
        self.image = image
        self.Refresh()

    def DrawBorder(self, dc):
        brush = wxBrush(wxNamedColour(self.back_color), wxSOLID)
        dc.SetBrush(brush)
        dc.SetPen(wxPen(wxNamedColour(self.border_color), 1))
        dc.DrawRectangle(0, 0, self.image_sizex, self.image_sizey)

    def DrawImage(self, dc):
        try:
            image = self.image
        except:
            return

        self.DrawBorder(dc)
        if image is None:
            return

        bmp = image.ConvertToBitmap()

        iwidth = bmp.GetWidth()   # dimensions of image file
        iheight = bmp.GetHeight()

        diffx = (self.image_sizex - iwidth)/2   # center calc
        if iwidth >= self.image_sizex -10:      # if image width fits in window adjust
            diffx = 5
            iwidth = self.image_sizex - 10

        diffy = (self.image_sizey - iheight)/2  # center calc
        if iheight >= self.image_sizey - 10:    # if image height fits in window adjust
            diffy = 5
            iheight = self.image_sizey - 10

        image.Rescale(iwidth, iheight)      # rescale to fit the window
        image.ConvertToBitmap()
        bmp = image.ConvertToBitmap()
        dc.DrawBitmap(bmp, diffx, diffy)        # draw the image to window


class ImageDialog(wxDialog):
    def __init__(self, parent, set_dir = None):
        wxDialog.__init__(self, parent, -1, "Image Browser", wxPyDefaultPosition, wxSize(400, 400))

        self.x_pos = 30     # initial display positions
        self.y_pos = 20
        self.delta = 20

        size = wxSize(80, 25)

        self.set_dir = os.getcwd()

        if set_dir != None:
            if os.path.exists(set_dir):     # set to working directory if nothing set
                self.set_dir = set_dir

        self.dir_x = self.x_pos
        self.dir_y = self.y_pos
        self.DisplayDir()       # display the directory value

        self.y_pos = self.y_pos + self.delta

        mID = wxNewId()
        wxButton(self, mID, ' Set Directory ', wxPoint(self.x_pos, self.y_pos), size).SetDefault()
        EVT_BUTTON(self, mID, self.SetDirect)

        self.type_posy = self.y_pos     # save the y position for the image type combo

        self.fl_ext = '*.bmp'   # initial setting for file filtering
        self.GetFiles()     # get the file list

        self.y_pos = self.y_pos + self.delta + 10

        self.list_height = 150

    # List of Labels
        mID = wxNewId()
        self.tb = tb = wxListBox(self, mID, wxPoint(self.x_pos, self.y_pos), wxSize(160, self.list_height), self.fl_list, wxLB_SINGLE)
        EVT_LISTBOX(self, mID, self.OnListClick)
        EVT_LISTBOX_DCLICK(self, mID, self.OnListDClick)

        width, height = self.tb.GetSizeTuple()
        image_posx = self.x_pos + width + 20       # positions for setting the image window
        image_posy = self.y_pos
        image_sizex = 150
        image_sizey = self.list_height

        self.fl_types = ["All Images", "Bmp", "Gif", "Png", "Jpg", "Ico", "Pnm", "Pcx", "Tif", "All Files"]
        self.fl_ext_types = { "All Images": "All", "Bmp": "*.bmp", "Gif": "*.gif", "Png": "*.png", "Jpg": "*.jpg",
                        "Ico": "*.ico", "Pnm": "*.pnm", "Pcx": "*.pcx", "Tif": "*.tif", "All Files": "*.*" }

        self.set_type = self.fl_types[0]    # initial file filter setting
        self.fl_ext = self.fl_ext_types[self.set_type]

        mID = wxNewId()
        self.sel_type = wxComboBox(self, mID, self.set_type, wxPoint(image_posx , self.type_posy), wxSize(150, -1), self.fl_types, wxCB_DROPDOWN)
        EVT_COMBOBOX(self, mID, self.OnSetType)

        self.image_view = ImageView(self, pos=wxPoint(image_posx, image_posy), size=wxSize(image_sizex, image_sizey))

        self.y_pos = self.y_pos + height + 20

        mID = wxNewId()
        wxButton(self, mID, ' Select ', wxPoint(100, self.y_pos), size).SetDefault()
        EVT_BUTTON(self, mID, self.OnOk)

        wxButton(self, wxID_CANCEL, 'Cancel', wxPoint(250, self.y_pos), size)

        self.y_pos = self.y_pos + self.delta
        fsize = wxSize(400, self.y_pos + 50)    # resize dialog for final vertical position
        self.SetSize(fsize)

        self.ResetFiles()

    def GetFiles(self):     # get the file list using directory and extension values
        if self.fl_ext == "All":
            all_files = []
            for ftypes in self.fl_types[1:-1]:    # get list of all available image types
                filter = self.fl_ext_types[ftypes]
                self.fl_val = FindFiles(self, self.set_dir, filter)
                all_files = all_files + self.fl_val.files   # add to list of files
            self.fl_list = all_files
        else:
            self.fl_val = FindFiles(self, self.set_dir, self.fl_ext)
            self.fl_list = self.fl_val.files

        self.fl_list.sort()     # sort the file list

    def DisplayDir(self):       # display the working directory
        wxStaticText(self, -1, self.set_dir, wxPoint(self.dir_x, self.dir_y), wxSize(250, -1))

    def OnSetType(self, event):
        val = event.GetString()      # get file type value
        self.fl_ext = self.fl_ext_types[val]
        self.ResetFiles()

    def OnListDClick(self, event):
        self.OnOk(0)

    def OnListClick(self, event):
        val = event.GetSelection()
        self.SetListValue(val)

    def SetListValue(self, val):
        file_nm = self.fl_list[val]
        self.set_file = file_val = os.path.join(self.set_dir, file_nm)
        self.image_view.SetValue(file_val)

    def SetDirect(self, event):     # set the new directory
        dlg = wxDirDialog(self)
        dlg.SetPath(self.set_dir)
        if dlg.ShowModal() == wxID_OK:
            self.set_dir = dlg.GetPath()
            self.ResetFiles()
        dlg.Destroy()

    def ResetFiles(self):   # refresh the display with files and initial image
        self.DisplayDir()
        self.GetFiles()
        self.tb.Set(self.fl_list)
        try:
            self.tb.SetSelection(0)
            self.SetListValue(0)
        except:
            self.image_view.SetValue(None)

    def GetFile(self):
        return self.set_file

    def GetDirectory(self):
        return self.set_dir

    def OnCancel(self, event):
        self.result = None
        self.EndModal(wxID_CANCEL)

    def OnOk(self, event):
        self.result = self.set_file
        self.EndModal(wxID_OK)


def OnFileDlg(self):
    dlg = wxFileDialog(self, "Choose an Image File", ".", "", "Bmp (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg", wxOPEN)
    if dlg.ShowModal() == wxID_OK:
        path = dlg.GetPath()
    else:
        path = None
    dlg.Destroy()
    return path

class FindFiles:
    def __init__(self, parent, dir, mask):
        filelist = []
        dirlist = [".."]
        self.dir = dir
        self.file = ""
        mask = mask.upper()
        pattern = self.MakeRegex(mask)
        for i in os.listdir(dir):
            if i == "." or i == "..":
                continue
            path = os.path.join(dir, i)
            path = path.upper()
            value = i.upper()

            if pattern.match(value) != None:
                filelist.append(i)

            self.files = filelist

    def MakeRegex(self, pattern):
        import re
        f = ""  # Set up a regex for file names
        for ch in pattern:
            if ch == "*":
                f = f + ".*"
            elif ch == ".":
                f = f + "\."
            elif ch == "?":
                f = f + "."
            else:
                f = f + ch

        return re.compile(f)

    def StripExt(self, file_nm):
        fl_fld = os.path.splitext(file_nm)
        fl_name = fl_fld[0]
        ext = fl_fld[1]
        return ext[1:]



"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Parameters.py,v 1.1.2.2 2003/03/20 23:14:27 RD Exp $"
__revision__ = "$Revision: 1.1.2.2 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


class _Param:
    """Used by this module to represent default wxPython parameter values,
    including parameter representations like style=wx.HSCROLL|wx.VSCROLL."""

    def __init__(self, value=None):
        if value is None:
            value = 'wx.' + self.__class__.__name__
        self.value = value

    def __repr__(self):
        return self.value

    def __or__(self, other):
        value = '%s|%s' % (self, other)
        return self.__class__(value)

_params = (
    'BOTH',
    'DEFAULT_FRAME_STYLE',
    'DefaultPosition',
    'DefaultSize',
    'DefaultValidator',
    'EmptyString',
    'EVT_NULL',
    'HSCROLL',
    'NO_BORDER',
    'NULL',
    'NullColour',
    'PyFrameNameStr',
    'PyNOTEBOOK_NAME',
    'PyPanelNameStr',
    'PyStatusLineNameStr',
    'PySTCNameStr',
    'PyToolBarNameStr',
    'SIZE_AUTO',
    'SIZE_USE_EXISTING',
    'ST_SIZEGRIP',
    'TAB_TRAVERSAL',
    'TB_HORIZONTAL',
    'VSCROLL',
    )

## Create classes, then instances, like this:

## class BOTH(Param): pass
## BOTH = BOTH()

for _param in _params:
    exec 'class %s(_Param): pass' % _param
    exec '%s = %s()' % (_param, _param)

del _param
del _params
del _Param



"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: ToolBar.py,v 1.1.2.1 2003/03/11 19:37:11 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
from Controls import Control
import Parameters as wx


class ToolBarBase(Control):
    """"""

    def AddCheckLabelTool(self):
        """"""
        pass

    def AddCheckTool(self):
        """"""
        pass

    def AddControl(self):
        """"""
        pass

    def AddLabelTool(self):
        """"""
        pass

    def AddRadioLabelTool(self):
        """"""
        pass

    def AddRadioTool(self):
        """"""
        pass

    def AddSeparator(self):
        """"""
        pass

    def AddSimpleTool(self):
        """"""
        pass

    def AddTool(self):
        """"""
        pass

    def ClearTools(self):
        """"""
        pass

    def DeleteTool(self):
        """"""
        pass

    def DeleteToolByPos(self):
        """"""
        pass

    def DoAddTool(self):
        """"""
        pass

    def DoInsertTool(self):
        """"""
        pass

    def EnableTool(self):
        """"""
        pass

    def FindControl(self):
        """"""
        pass

    def FindToolForPosition(self):
        """"""
        pass

    def GetMargins(self):
        """"""
        pass

    def GetMaxCols(self):
        """"""
        pass

    def GetMaxRows(self):
        """"""
        pass

    def GetToolBitmapSize(self):
        """"""
        pass

    def GetToolClientData(self):
        """"""
        pass

    def GetToolEnabled(self):
        """"""
        pass

    def GetToolLongHelp(self):
        """"""
        pass

    def GetToolMargins(self):
        """"""
        pass

    def GetToolPacking(self):
        """"""
        pass

    def GetToolSeparation(self):
        """"""
        pass

    def GetToolShortHelp(self):
        """"""
        pass

    def GetToolSize(self):
        """"""
        pass

    def GetToolState(self):
        """"""
        pass

    def InsertControl(self):
        """"""
        pass

    def InsertLabelTool(self):
        """"""
        pass

    def InsertSeparator(self):
        """"""
        pass

    def InsertSimpleTool(self):
        """"""
        pass

    def InsertTool(self):
        """"""
        pass

    def IsVertical(self):
        """"""
        pass

    def Realize(self):
        """"""
        pass

    def RemoveTool(self):
        """"""
        pass

    def SetMargins(self):
        """"""
        pass

    def SetMarginsXY(self):
        """"""
        pass

    def SetMaxRowsCols(self):
        """"""
        pass

    def SetRows(self):
        """"""
        pass

    def SetToggle(self):
        """"""
        pass

    def SetToolBitmapSize(self):
        """"""
        pass

    def SetToolClientData(self):
        """"""
        pass

    def SetToolLongHelp(self):
        """"""
        pass

    def SetToolPacking(self):
        """"""
        pass

    def SetToolSeparation(self):
        """"""
        pass

    def SetToolShortHelp(self):
        """"""
        pass

    def ToggleTool(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ToolBar(ToolBarBase):
    """"""

    def Create(self):
        """"""
        pass

    def FindToolForPosition(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ToolBarSimple(ToolBarBase):
    """"""

    def Create(self):
        """"""
        pass

    def FindToolForPosition(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass


class ToolBarToolBase(Object):
    """"""

    def Attach(self):
        """"""
        pass

    def CanBeToggled(self):
        """"""
        pass

    def Destroy(self):
        """"""
        pass

    def Detach(self):
        """"""
        pass

    def Enable(self):
        """"""
        pass

    def GetBitmap(self):
        """"""
        pass

    def GetBitmap1(self):
        """"""
        pass

    def GetBitmap2(self):
        """"""
        pass

    def GetClientData(self):
        """"""
        pass

    def GetControl(self):
        """"""
        pass

    def GetDisabledBitmap(self):
        """"""
        pass

    def GetId(self):
        """"""
        pass

    def GetKind(self):
        """"""
        pass

    def GetLabel(self):
        """"""
        pass

    def GetLongHelp(self):
        """"""
        pass

    def GetNormalBitmap(self):
        """"""
        pass

    def GetShortHelp(self):
        """"""
        pass

    def GetStyle(self):
        """"""
        pass

    def GetToolBar(self):
        """"""
        pass

    def IsButton(self):
        """"""
        pass

    def IsControl(self):
        """"""
        pass

    def IsEnabled(self):
        """"""
        pass

    def IsSeparator(self):
        """"""
        pass

    def IsToggled(self):
        """"""
        pass

    def SetBitmap1(self):
        """"""
        pass

    def SetBitmap2(self):
        """"""
        pass

    def SetClientData(self):
        """"""
        pass

    def SetDisabledBitmap(self):
        """"""
        pass

    def SetLabel(self):
        """"""
        pass

    def SetLongHelp(self):
        """"""
        pass

    def SetNormalBitmap(self):
        """"""
        pass

    def SetShortHelp(self):
        """"""
        pass

    def SetToggle(self):
        """"""
        pass

    def Toggle(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass





#----------------------------------------------------------------------
# Name:        wxPython.lib.stattext
# Purpose:     A generic wxGenStaticText class.  Using this should
#              eliminate some of the platform differences in wxStaticText,
#              such as background colours and mouse sensitivity.
#
# Author:      Robin Dunn
#
# Created:     8-July-2002
# RCS-ID:      $Id: stattext.py,v 1.2.2.6 2003/02/26 18:38:14 RD Exp $
# Copyright:   (c) 2002 by Total Control Software
# Licence:     wxWindows license
#----------------------------------------------------------------------

"""
"""

from wxPython.wx import *

#----------------------------------------------------------------------

class wxGenStaticText(wxPyControl):
    labelDelta = 1

    def __init__(self, parent, ID, label,
                 pos = wxDefaultPosition, size = wxDefaultSize,
                 style = 0,
                 name = "genstattext"):
        wxPyControl.__init__(self, parent, ID, pos, size, style, wxDefaultValidator, name)

        wxPyControl.SetLabel(self, label) # don't check wxST_NO_AUTORESIZE yet
        self.SetPosition(pos)
        font = parent.GetFont()
        if not font.Ok():
            font = wxSystemSettings_GetSystemFont(wxSYS_DEFAULT_GUI_FONT)
        wxPyControl.SetFont(self, font) # same here

        clr = parent.GetBackgroundColour()
        if not clr.Ok():
            clr = wxSystemSettings_GetSystemColour(wxSYS_COLOUR_BTNFACE)
        self.SetBackgroundColour(clr)

        clr = parent.GetForegroundColour()
        if not clr.Ok():
            clr = wxSystemSettings_GetSystemColour(wxSYS_COLOUR_BTNTEXT)
        self.SetForegroundColour(clr)

        rw, rh = size
        bw, bh = self.GetBestSize()
        if rw == -1: rw = bw
        if rh == -1: rh = bh
        self.SetSize(wxSize(rw, rh))

        EVT_ERASE_BACKGROUND(self, self.OnEraseBackground)
        EVT_PAINT(self,            self.OnPaint)


    def SetLabel(self, label):
        """
        Sets the static text label and updates the control's size to exactly
        fit the label unless the control has wxST_NO_AUTORESIZE flag.
        """
        wxPyControl.SetLabel(self, label)
        style = self.GetWindowStyleFlag()
        if not style & wxST_NO_AUTORESIZE:
            self.SetSize(self.GetBestSize())
        self.Refresh()


    def SetFont(self, font):
        """
        Sets the static text font and updates the control's size to exactly
        fit the label unless the control has wxST_NO_AUTORESIZE flag.
        """
        wxPyControl.SetFont(self, font)
        style = self.GetWindowStyleFlag()
        if not style & wxST_NO_AUTORESIZE:
            self.SetSize(self.GetBestSize())
        self.Refresh()


    def DoGetBestSize(self):
        """Overridden base class virtual.  Determines the best size of the
        button based on the label size."""
        label = self.GetLabel()
        maxWidth = totalHeight = 0
        for line in label.split('\n'):
            if line == '':
                w, h = self.GetTextExtent('W')  # empty lines have height too
            else:
                w, h = self.GetTextExtent(line)
            totalHeight += h
            maxWidth = max(maxWidth, w)
        return wxSize(maxWidth, totalHeight)


    def AcceptsFocus(self):
        """Overridden base class virtual."""
        return False


    def OnPaint(self, event):
        dc = wxBufferedPaintDC(self)
        #dc = wxPaintDC(self)
        width, height = self.GetClientSize()
        if not width or not height:
            return
        dc.SetBackground(wxBrush(self.GetBackgroundColour(), wxSOLID))
        dc.SetTextForeground(self.GetForegroundColour())
        dc.Clear()
        dc.SetFont(self.GetFont())
        label = self.GetLabel()
        style = self.GetWindowStyleFlag()
        x = y = 0
        for line in label.split('\n'):
            if line == '':
                w, h = self.GetTextExtent('W')  # empty lines have height too
            else:
                w, h = self.GetTextExtent(line)
            if style & wxALIGN_RIGHT:
                x = width - w
            if style & wxALIGN_CENTER:
                x = (width - w)/2
            dc.DrawText(line, x, y)
            y += h


    def OnEraseBackground(self, event):
        pass




#----------------------------------------------------------------------





"""
wxMVCTree is a control which handles hierarchical data. It is constructed
in model-view-controller architecture, so the display of that data, and
the content of the data can be changed greatly without affecting the other parts.

wxMVCTree actually is even more configurable than MVC normally implies, because
almost every aspect of it is pluggable:
    wxMVCTree - Overall controller, and the window that actually gets placed
    in the GUI.
        Painter - Paints the control. The 'view' part of MVC.
           NodePainter - Paints just the nodes
           LinePainter - Paints just the lines between the nodes
           TextConverter - Figures out what text to print for each node
        Editor - Edits the contents of a node, if the model is editable.
        LayoutEngine - Determines initial placement of nodes
        Transform - Adjusts positions of nodes for movement or special effects.
        TreeModel - Contains the data which the rest of the control acts
        on. The 'model' part of MVC.

Author/Maintainer - Bryn Keller <xoltar@starship.python.net>


NOTE: This module is *not* supported in any way.  Use it however you
      wish, but be warned that dealing with any consequences is
      entirly up to you.
      --Robin
"""

#------------------------------------------------------------------------
from wxPython.wx import *
import os, sys, traceback
#------------------------------------------------------------------------

class MVCTreeNode:
    """
    Used internally by wxMVCTree to manage its data. Contains information about
    screen placement, the actual data associated with it, and more. These are
    the nodes passed to all the other helper parts to do their work with.
    """
    def __init__(self, data=None, parent = None, kids = None, x = 0, y = 0):
        self.x = 0
        self.y = 0
        self.projx = 0
        self.projy = 0
        self.parent = parent
        self.kids = kids
        if self.kids is None:
            self.kids = []
        self.data = data
        self.expanded = False
        self.selected = False
        self.built = False
        self.scale = 0

    def GetChildren(self):
        return self.kids

    def GetParent(self):
        return self.parent

    def Remove(self, node):
        try:
            self.kids.remove(node)
        except:
            pass
    def Add(self, node):
        self.kids.append(node)
        node.SetParent(self)

    def SetParent(self, parent):
        if self.parent and not (self.parent is parent):
            self.parent.Remove(self)
        self.parent = parent
    def __str__(self):
        return "Node: "  + str(self.data) + " (" + str(self.x) + ", " + str(self.y) + ")"
    def __repr__(self):
        return str(self.data)
    def GetTreeString(self, tabs=0):
        s = tabs * '\t' + str(self) + '\n'
        for kid in self.kids:
            s = s + kid.GetTreeString(tabs + 1)
        return s


class Editor:
    def __init__(self, tree):
        self.tree = tree
    def Edit(self, node):
        raise NotImplementedError
    def EndEdit(self, node, commit):
        raise NotImplementedError
    def CanEdit(self, node):
        raise NotImplementedError

class LayoutEngine:
    """
    Interface for layout engines.
    """
    def __init__(self, tree):
        self.tree = tree
    def Layout(self, node):
        raise NotImplementedError
    def GetNodeList(self):
        raise NotImplementedError

class Transform:
    """
    Transform interface.
    """
    def __init__(self, tree):
        self.tree = tree
    def Transform(self, node, offset, rotation):
        """
        This method should only change the projx and projy attributes of
        the node. These represent the position of the node as it should
        be drawn on screen. Adjusting the x and y attributes can and
        should cause havoc.
        """
        raise NotImplementedError

    def GetSize(self):
        """
        Returns the size of the entire tree as laid out and transformed
        as a tuple
        """
        raise NotImplementedError

class Painter:
    """
    This is the interface that wxMVCTree expects from painters. All painters should
    be Painter subclasses.
    """
    def __init__(self, tree):
        self.tree = tree
        self.textcolor = wxNamedColour("BLACK")
        self.bgcolor = wxNamedColour("WHITE")
        self.fgcolor = wxNamedColour("BLUE")
        self.linecolor = wxNamedColour("GREY")
        self.font = wxFont(9, wxDEFAULT, wxNORMAL, wxNORMAL, False)
        self.bmp = None

    def GetFont(self):
        return self.font

    def SetFont(self, font):
        self.font = font
        self.tree.Refresh()
    def GetBuffer(self):
        return self.bmp
    def ClearBuffer(self):
        self.bmp = None
    def Paint(self, dc, node, doubleBuffered=1, paintBackground=1):
        raise NotImplementedError
    def GetTextColour(self):
        return self.textcolor
    def SetTextColour(self, color):
        self.textcolor = color
        self.textbrush = wxBrush(color)
        self.textpen = wxPen(color, 1, wxSOLID)
    def GetBackgroundColour(self):
        return self.bgcolor
    def SetBackgroundColour(self, color):
        self.bgcolor = color
        self.bgbrush = wxBrush(color)
        self.bgpen = wxPen(color, 1, wxSOLID)
    def GetForegroundColour(self):
        return self.fgcolor
    def SetForegroundColour(self, color):
        self.fgcolor = color
        self.fgbrush = wxBrush(color)
        self.fgpen = wxPen(color, 1, wxSOLID)
    def GetLineColour(self):
        return self.linecolor
    def SetLineColour(self, color):
        self.linecolor = color
        self.linebrush = wxBrush(color)
        self.linepen = wxPen( color, 1, wxSOLID)
    def GetForegroundPen(self):
        return self.fgpen
    def GetBackgroundPen(self):
        return self.bgpen
    def GetTextPen(self):
        return self.textpen
    def GetForegroundBrush(self):
        return self.fgbrush
    def GetBackgroundBrush(self):
        return self.bgbrush
    def GetTextBrush(self):
        return self.textbrush
    def GetLinePen(self):
        return self.linepen
    def GetLineBrush(self):
        return self.linebrush
    def OnMouse(self, evt):
        if evt.LeftDClick():
            x, y = self.tree.CalcUnscrolledPosition(evt.GetX(), evt.GetY())
            for item in self.rectangles:
                if item[1].Contains((x,y)):
                    self.tree.Edit(item[0].data)
                    self.tree.OnNodeClick(item[0], evt)
                    return
        elif evt.ButtonDown():
            x, y = self.tree.CalcUnscrolledPosition(evt.GetX(), evt.GetY())
            for item in self.rectangles:
                if item[1].Contains((x, y)):
                    self.tree.OnNodeClick(item[0], evt)
                    return
            for item in self.knobs:
                if item[1].Contains((x, y)):
                    self.tree.OnKnobClick(item[0])
                    return
        evt.Skip()


class wxTreeModel:
    """
    Interface for tree models
    """
    def GetRoot(self):
        raise NotImplementedError
    def SetRoot(self, root):
        raise NotImplementedError
    def GetChildCount(self, node):
        raise NotImplementedError
    def GetChildAt(self, node, index):
        raise NotImplementedError
    def GetParent(self, node):
        raise NotImplementedError
    def AddChild(self, parent, child):
        if hasattr(self, 'tree') and self.tree:
            self.tree.NodeAdded(parent, child)
    def RemoveNode(self, child):
        if hasattr(self, 'tree') and self.tree:
            self.tree.NodeRemoved(child)
    def InsertChild(self, parent, child, index):
        if hasattr(self, 'tree') and self.tree:
            self.tree.NodeInserted(parent, child, index)
    def IsLeaf(self, node):
        raise NotImplementedError

    def IsEditable(self, node):
        return False

    def SetEditable(self, node):
        return False

class NodePainter:
    """
    This is the interface expected of a nodepainter.
    """
    def __init__(self, painter):
        self.painter = painter
    def Paint(self, node, dc, location = None):
        """
        location should be provided only to draw in an unusual position
        (not the node's normal position), otherwise the node's projected x and y
        coordinates will be used.
        """
        raise NotImplementedError

class LinePainter:
    """
    The linepainter interface.
    """
    def __init__(self, painter):
        self.painter = painter
    def Paint(self, parent, child, dc):
        raise NotImplementedError

class TextConverter:
    """
    TextConverter interface.
    """
    def __init__(self, painter):
        self.painter = painter
    def Convert(node):
        """
        Should return a string. The node argument will be an
        MVCTreeNode.
        """
        raise NotImplementedError


class BasicTreeModel(wxTreeModel):
    """
    A very simple treemodel implementation, but flexible enough for many needs.
    """
    def __init__(self):
        self.children = {}
        self.parents = {}
        self.root = None
    def GetRoot(self):
        return self.root
    def SetRoot(self, root):
        self.root = root
    def GetChildCount(self, node):
        if self.children.has_key(node):
            return len(self.children[node])
        else:
            return 0
    def GetChildAt(self, node, index):
        return self.children[node][index]

    def GetParent(self, node):
        return self.parents[node]

    def AddChild(self, parent, child):
        self.parents[child]=parent
        if not self.children.has_key(parent):
            self.children[parent]=[]
        self.children[parent].append(child)
        wxTreeModel.AddChild(self, parent, child)
        return child

    def RemoveNode(self, node):
        parent = self.parents[node]
        del self.parents[node]
        self.children[parent].remove(node)
        wxTreeModel.RemoveNode(self, node)

    def InsertChild(self, parent, child, index):
        self.parents[child]=parent
        if not self.children.has_key(parent):
            self.children[parent]=[]
        self.children[parent].insert(child, index)
        wxTreeModel.InsertChild(self, parent, child, index)
        return child

    def IsLeaf(self, node):
        return not self.children.has_key(node)

    def IsEditable(self, node):
        return False

    def SetEditable(self, node, bool):
        return False


class FileEditor(Editor):
    def Edit(self, node):
        treenode = self.tree.nodemap[node]
        self.editcomp = wxTextCtrl(self.tree, -1)
        for rect in self.tree.painter.rectangles:
            if rect[0] == treenode:
                self.editcomp.SetPosition((rect[1][0], rect[1][1]))
                break
        self.editcomp.SetValue(node.fileName)
        self.editcomp.SetSelection(0, len(node.fileName))
        self.editcomp.SetFocus()
        self.treenode = treenode
#        EVT_KEY_DOWN(self.editcomp, self._key)
        EVT_KEY_UP(self.editcomp, self._key)
        EVT_LEFT_DOWN(self.editcomp, self._mdown)
        self.editcomp.CaptureMouse()

    def CanEdit(self, node):
        return isinstance(node, FileWrapper)

    def EndEdit(self, commit):
        if not self.tree._EditEnding(self.treenode.data):
            return
        if commit:
            node = self.treenode.data
            try:
                os.rename(node.path + os.sep + node.fileName, node.path + os.sep + self.editcomp.GetValue())
                node.fileName = self.editcomp.GetValue()
            except:
                traceback.print_exc()
        self.editcomp.ReleaseMouse()
        self.editcomp.Destroy()
        del self.editcomp
        self.tree.Refresh()


    def _key(self, evt):
        if evt.KeyCode() == WXK_RETURN:
            self.EndEdit(True)
        elif evt.KeyCode() == WXK_ESCAPE:
            self.EndEdit(False)
        else:
            evt.Skip()

    def _mdown(self, evt):
        if evt.IsButton():
            pos = evt.GetPosition()
            edsize = self.editcomp.GetSize()
            if pos.x < 0 or pos.y < 0 or pos.x > edsize.width or pos.y > edsize.height:
                self.EndEdit(False)


class FileWrapper:
    """
    Node class for FSTreeModel.
    """
    def __init__(self, path, fileName):
        self.path = path
        self.fileName = fileName

    def __str__(self):
        return self.fileName

class FSTreeModel(BasicTreeModel):
    """
    This treemodel models the filesystem starting from a given path.
    """
    def __init__(self, path):
        BasicTreeModel.__init__(self)
        fw = FileWrapper(path, path.split(os.sep)[-1])
        self._Build(path, fw)
        self.SetRoot(fw)
        self._editable = True
    def _Build(self, path, fileWrapper):
        for name in os.listdir(path):
            fw = FileWrapper(path, name)
            self.AddChild(fileWrapper, fw)
            childName = path + os.sep + name
            if os.path.isdir(childName):
                self._Build(childName, fw)

    def IsEditable(self, node):
        return self._editable

    def SetEditable(self, node, bool):
        self._editable = bool

class LateFSTreeModel(FSTreeModel):
    """
    This treemodel models the filesystem starting from a given path.
    It retrieves the directory list as requested.
    """
    def __init__(self, path):
        BasicTreeModel.__init__(self)
        name = path.split(os.sep)[-1]
        pathpart = path[:-len(name)]
        fw = FileWrapper(pathpart, name)
        self._Build(path, fw)
        self.SetRoot(fw)
        self._editable = True
        self.children = {}
        self.parents = {}
    def _Build(self, path, parent):
        ppath = parent.path + os.sep + parent.fileName
        if not os.path.isdir(ppath):
            return
        for name in os.listdir(ppath):
            fw = FileWrapper(ppath, name)
            self.AddChild(parent, fw)
    def GetChildCount(self, node):
        if self.children.has_key(node):
            return FSTreeModel.GetChildCount(self, node)
        else:
            self._Build(node.path, node)
            return FSTreeModel.GetChildCount(self, node)

    def IsLeaf(self, node):
        return not os.path.isdir(node.path + os.sep + node.fileName)

class StrTextConverter(TextConverter):
    def Convert(self, node):
        return str(node.data)

class NullTransform(Transform):
    def GetSize(self):
        return tuple(self.size)

    def Transform(self, node, offset, rotation):
        self.size = [0,0]
        list = self.tree.GetLayoutEngine().GetNodeList()
        for node in list:
            node.projx = node.x + offset[0]
            node.projy = node.y + offset[1]
            if node.projx > self.size[0]:
                self.size[0] = node.projx
            if node.projy > self.size[1]:
                self.size[1] = node.projy

class Rect:
    def __init__(self, x, y, width, height):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
    def __getitem__(self, index):
        return (self.x, self.y, self.width, self.height)[index]

    def __setitem__(self, index, value):
        name = ['x', 'y', 'width', 'height'][index]
        setattr(self, name, value)

    def Contains(self, other):
        if type(other) == type(()):
            other = Rect(other[0], other[1], 0, 0)
        if other.x >= self.x:
            if other.y >= self.y:
                if other.width + other.x <= self.width + self.x:
                    if other.height + other.y  <= self.height + self.y:
                        return True
        return False

    def __str__(self):
        return "Rect: " + str([self.x, self.y, self.width, self.height])

class TreeLayout(LayoutEngine):
    def SetHeight(self, num):
        self.NODE_HEIGHT = num

    def __init__(self, tree):
        LayoutEngine.__init__(self, tree)
        self.NODE_STEP = 20
        self.NODE_HEIGHT = 20
        self.nodelist = []

    def Layout(self, node):
        self.nodelist = []
        self.NODE_HEIGHT = self.tree.GetFont().GetPointSize() * 2
        self.layoutwalk(node)

    def GetNodeList(self):
        return self.nodelist

    def layoutwalk(self, node):
        if node == self.tree.currentRoot:
            node.level = 1
            self.lastY = (-self.NODE_HEIGHT)
        node.x = self.NODE_STEP * node.level
        node.y = self.lastY + self.NODE_HEIGHT
        self.lastY = node.y
        self.nodelist.append(node)
        if node.expanded:
            for kid in node.kids:
                kid.level = node.level + 1
                self.layoutwalk(kid)

class TreePainter(Painter):
    """
    The default painter class. Uses double-buffering, delegates the painting of nodes and
    lines to helper classes deriving from NodePainter and LinePainter.
    """
    def __init__(self, tree, nodePainter = None, linePainter = None, textConverter = None):
        Painter.__init__(self, tree)
        if not nodePainter:
            nodePainter = TreeNodePainter(self)
        self.nodePainter = nodePainter
        if not linePainter:
            linePainter = TreeLinePainter(self)
        self.linePainter = linePainter
        if not textConverter:
            textConverter = StrTextConverter(self)
        self.textConverter = textConverter
        self.charWidths = []

    def Paint(self, dc, node, doubleBuffered=1, paintBackground=1):
        if not self.charWidths:
            self.charWidths = []
            for i in range(25):
                self.charWidths.append(dc.GetTextExtent("D")[0] * i)
            self.charHeight = dc.GetTextExtent("D")[1]
        self.textpen = wxPen(self.GetTextColour(), 1, wxSOLID)
        self.fgpen = wxPen(self.GetForegroundColour(), 1, wxSOLID)
        self.bgpen = wxPen(self.GetBackgroundColour(), 1, wxSOLID)
        self.linepen = wxPen(self.GetLineColour(), 1, wxSOLID)
        self.dashpen = wxPen(self.GetLineColour(), 1, wxDOT)
        self.textbrush = wxBrush(self.GetTextColour(), wxSOLID)
        self.fgbrush = wxBrush(self.GetForegroundColour(), wxSOLID)
        self.bgbrush = wxBrush(self.GetBackgroundColour(), wxSOLID)
        self.linebrush = wxPen(self.GetLineColour(), 1, wxSOLID)
        treesize = self.tree.GetSize()
        size = self.tree.transform.GetSize()
        size = (max(treesize.width, size[0]+50), max(treesize.height, size[1]+50))
        dc.BeginDrawing()
        if doubleBuffered:
            mem_dc = wxMemoryDC()
            if not self.GetBuffer():
                self.knobs = []
                self.rectangles = []
                self.bmp = wxEmptyBitmap(size[0], size[1])
                mem_dc.SelectObject(self.GetBuffer())
                mem_dc.SetPen(self.GetBackgroundPen())
                mem_dc.SetBrush(self.GetBackgroundBrush())
                mem_dc.DrawRectangle(0, 0, size[0], size[1])
                mem_dc.SetFont(self.tree.GetFont())
                self.paintWalk(node, mem_dc)
            else:
                mem_dc.SelectObject(self.GetBuffer())
            xstart, ystart = self.tree.CalcUnscrolledPosition(0,0)
            size = self.tree.GetClientSizeTuple()
            dc.Blit(xstart, ystart, size[0], size[1], mem_dc, xstart, ystart)
        else:
            if node == self.tree.currentRoot:
                self.knobs = []
                self.rectangles = []
            dc.SetPen(self.GetBackgroundPen())
            dc.SetBrush(self.GetBackgroundBrush())
            dc.SetFont(self.tree.GetFont())
            if paintBackground:
                dc.DrawRectangle(0, 0, size[0], size[1])
            if node:
                #Call with not paintBackground because if we are told not to paint the
                #whole background, we have to paint in parts to undo selection coloring.
                pb = paintBackground
                self.paintWalk(node, dc, not pb)
        dc.EndDrawing()

    def GetDashPen(self):
        return self.dashpen

    def SetLinePen(self, pen):
        Painter.SetLinePen(self, pen)
        self.dashpen = wxPen(pen.GetColour(), 1, wxDOT)

    def paintWalk(self, node, dc, paintRects=0):
        self.linePainter.Paint(node.parent, node, dc)
        self.nodePainter.Paint(node, dc, drawRects = paintRects)
        if node.expanded:
            for kid in node.kids:
                if not self.paintWalk(kid, dc, paintRects):
                    return False
            for kid in node.kids:
                px = (kid.projx - self.tree.layout.NODE_STEP) + 5
                py = kid.projy + kid.height/2
                if (not self.tree.model.IsLeaf(kid.data)) or ((kid.expanded or self.tree._assumeChildren) and len(kid.kids)):
                    dc.SetPen(self.linepen)
                    dc.SetBrush(self.bgbrush)
                    dc.DrawRectangle(px -4, py-4, 9, 9)
                    self.knobs.append( (kid, Rect(px -4, py -4, 9, 9)) )
                    dc.SetPen(self.textpen)
                    if not kid.expanded:
                        dc.DrawLine(px, py -2, px, py + 3)
                    dc.DrawLine(px -2, py, px + 3, py)
        if node == self.tree.currentRoot:
            px = (node.projx - self.tree.layout.NODE_STEP) + 5
            py = node.projy + node.height/2
            dc.SetPen(self.linepen)
            dc.SetBrush(self.bgbrush)
            dc.DrawRectangle(px -4, py-4, 9, 9)
            self.knobs.append( (node, Rect(px -4, py -4, 9, 9)) )
            dc.SetPen(self.textpen)
            if not node.expanded:
                dc.DrawLine(px, py -2, px, py + 3)
            dc.DrawLine(px -2, py, px + 3, py)
        return True

    def OnMouse(self, evt):
        Painter.OnMouse(self, evt)

class TreeNodePainter(NodePainter):
    def Paint(self, node, dc, location = None, drawRects = 0):
        text = self.painter.textConverter.Convert(node)
        extent = dc.GetTextExtent(text)
        node.width = extent[0]
        node.height = extent[1]
        if node.selected:
            dc.SetPen(self.painter.GetLinePen())
            dc.SetBrush(self.painter.GetForegroundBrush())
            dc.SetTextForeground(wxNamedColour("WHITE"))
            dc.DrawRectangle(node.projx -1, node.projy -1, node.width + 3, node.height + 3)
        else:
            if drawRects:
                dc.SetBrush(self.painter.GetBackgroundBrush())
                dc.SetPen(self.painter.GetBackgroundPen())
                dc.DrawRectangle(node.projx -1, node.projy -1, node.width + 3, node.height + 3)
            dc.SetTextForeground(self.painter.GetTextColour())
        dc.DrawText(text, node.projx, node.projy)
        self.painter.rectangles.append((node, Rect(node.projx, node.projy, node.width, node.height)))

class TreeLinePainter(LinePainter):
    def Paint(self, parent, child, dc):
        dc.SetPen(self.painter.GetDashPen())
        px = py = cx = cy = 0
        if parent is None or child == self.painter.tree.currentRoot:
            px = (child.projx - self.painter.tree.layout.NODE_STEP) + 5
            py = child.projy + self.painter.tree.layout.NODE_HEIGHT/2 -2
            cx = child.projx
            cy = py
            dc.DrawLine(px, py, cx, cy)
        else:
            px = parent.projx + 5
            py = parent.projy + parent.height
            cx = child.projx -5
            cy = child.projy + self.painter.tree.layout.NODE_HEIGHT/2 -3
            dc.DrawLine(px, py, px, cy)
            dc.DrawLine(px, cy, cx, cy)


wxEVT_MVCTREE_BEGIN_EDIT = 20204 #Start editing. Vetoable.
wxEVT_MVCTREE_END_EDIT = 20205 #Stop editing. Vetoable.
wxEVT_MVCTREE_DELETE_ITEM = 20206 #Item removed from model.
wxEVT_MVCTREE_ITEM_EXPANDED = 20209
wxEVT_MVCTREE_ITEM_EXPANDING = 20210
wxEVT_MVCTREE_ITEM_COLLAPSED = 20211
wxEVT_MVCTREE_ITEM_COLLAPSING = 20212
wxEVT_MVCTREE_SEL_CHANGED = 20213
wxEVT_MVCTREE_SEL_CHANGING = 20214 #Vetoable.
wxEVT_MVCTREE_KEY_DOWN = 20215
wxEVT_MVCTREE_ADD_ITEM = 20216 #Item added to model.

def EVT_MVCTREE_SEL_CHANGED(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_SEL_CHANGED, func)

def EVT_MVCTREE_SEL_CHANGING(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_SEL_CHANGING, func)

def EVT_MVCTREE_ITEM_EXPANDED(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_ITEM_EXPANDED, func)

def EVT_MVCTREE_ITEM_EXPANDING(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_ITEM_EXPANDING, func)

def EVT_MVCTREE_ITEM_COLLAPSED(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_ITEM_COLLAPSED, func)

def EVT_MVCTREE_ITEM_COLLAPSING(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_ITEM_COLLAPSING, func)

def EVT_MVCTREE_ADD_ITEM(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_ADD_ITEM, func)

def EVT_MVCTREE_DELETE_ITEM(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_DELETE_ITEM, func)

def EVT_MVCTREE_KEY_DOWN(win, id, func):
    win.Connect(id, -1, wxEVT_MVCTREE_KEY_DOWN, func)


class wxMVCTreeEvent(wxPyCommandEvent):
    def __init__(self, type, id, node = None, nodes = None, keyEvent = None, **kwargs):
        apply(wxPyCommandEvent.__init__, (self, type, id), kwargs)
        self.node = node
        self.nodes = nodes
        self.keyEvent = keyEvent
    def GetNode(self):
        return self.node
    def GetNodes(self):
        return self.nodes
    def getKeyEvent(self):
        return self.keyEvent

class wxMVCTreeNotifyEvent(wxMVCTreeEvent):
    def __init__(self, type, id, node = None, nodes = None, **kwargs):
        apply(wxMVCTreeEvent.__init__, (self, type, id, node, nodes), kwargs)
        self.notify = wxNotifyEvent(type, id)
    def getNotifyEvent(self):
        return self.notify

class wxMVCTree(wxScrolledWindow):
    """
    The main mvc tree class.
    """
    def __init__(self, parent, id, model = None, layout = None, transform = None,
                 painter = None, *args, **kwargs):
        apply(wxScrolledWindow.__init__, (self, parent, id), kwargs)
        self.nodemap = {}
        self._multiselect = False
        self._selections = []
        self._assumeChildren = False
        self._scrollx = False
        self._scrolly = False
        self.doubleBuffered = False
        self._lastPhysicalSize = self.GetSize()
        self._editors = []
        if not model:
            model = BasicTreeModel()
            model.SetRoot("Root")
        self.SetModel(model)
        if not layout:
            layout = TreeLayout(self)
        self.layout = layout
        if not transform:
           transform = NullTransform(self)
        self.transform = transform
        if not painter:
            painter = TreePainter(self)
        self.painter = painter
        self.SetFont(wxFont(9, wxDEFAULT, wxNORMAL, wxNORMAL, False))
        EVT_MOUSE_EVENTS(self, self.OnMouse)
        EVT_KEY_DOWN(self, self.OnKeyDown)
        self.doubleBuffered = True
        EVT_SIZE(self, self.OnSize)
        EVT_ERASE_BACKGROUND(self, self.OnEraseBackground)
        EVT_PAINT(self, self.OnPaint)


    def Refresh(self):
        if self.doubleBuffered:
            self.painter.ClearBuffer()
        wxScrolledWindow.Refresh(self, False)

    def GetPainter(self):
        return self.painter

    def GetLayoutEngine(self):
        return self.layout

    def GetTransform(self):
        return self.transform

    def __repr__(self):
        return "<wxMVCTree instance at %s>" % str(hex(id(self)))

    def __str__(self):
        return self.__repr__()

    def NodeAdded(self, parent, child):
        e = wxMVCTreeEvent(wxEVT_MVCTREE_ADD_ITEM, self.GetId(), node = child, nodes = [parent, child])
        self.GetEventHandler().ProcessEvent(e)
        self.painter.ClearBuffer()

    def NodeInserted(self, parent, child, index):
        e = wxMVCTreeEvent(wxEVT_MVCTREE_ADD_ITEM, self.GetId(), node = child, nodes = [parent, child])
        self.GetEventHandler().ProcessEvent(e)
        self.painter.ClearBuffer()

    def NodeRemoved(self, node):
        e = wxMVCTreeEvent(wxEVT_MVCTREE_DELETE_ITEM, self.GetId(), node = child, nodes = [parent, child])
        self.GetEventHandler().ProcessEvent(e)
        self.painter.ClearBuffer()

    def OnKeyDown(self, evt):
        e = wxMVCTreeEvent(wxEVT_MVCTREE_KEY_DOWN, self.GetId(), keyEvent = evt)
        self.GetEventHandler().ProcessEvent(e)

    def SetFont(self, font):
        self.painter.SetFont(font)
        dc = wxClientDC(self)
        dc.SetFont(font)
        self.layout.SetHeight(dc.GetTextExtent("")[1] + 18)
        self.painter.ClearBuffer()

    def GetFont(self):
        return self.painter.GetFont()

    def AddEditor(self, editor):
        self._editors.append(editor)

    def RemoveEditor(self, editor):
        self._editors.remove(editor)

    def OnMouse(self, evt):
        self.painter.OnMouse(evt)

    def OnNodeClick(self, node, mouseEvent):
        if node.selected and (self.IsMultiSelect() and mouseEvent.ControlDown()):
            self.RemoveFromSelection(node.data)
        else:
            self.AddToSelection(node.data, mouseEvent.ControlDown(), mouseEvent.ShiftDown())

    def OnKnobClick(self, node):
        self.SetExpanded(node.data, not node.expanded)

    def GetDisplayText(self, node):
        treenode = self.nodemap[node]
        return self.painter.textConverter.Convert(treenode)

    def IsDoubleBuffered(self):
        return self.doubleBuffered

    def SetDoubleBuffered(self, bool):
        """
        By default wxMVCTree is double-buffered.
        """
        self.doubleBuffered = bool

    def GetModel(self):
        return self.model

    def SetModel(self, model):
        """
        Completely change the data to be displayed.
        """
        self.model = model
        model.tree = self
        self.laidOut = 0
        self.transformed = 0
        self._selections = []
        self.layoutRoot = MVCTreeNode()
        self.layoutRoot.data = self.model.GetRoot()
        self.layoutRoot.expanded = True
        self.LoadChildren(self.layoutRoot)
        self.currentRoot = self.layoutRoot
        self.offset = [0,0]
        self.rotation = 0
        self._scrollset = None
        self.Refresh()

    def GetCurrentRoot(self):
        return self.currentRoot

    def LoadChildren(self, layoutNode):
        if layoutNode.built:
            return
        else:
            self.nodemap[layoutNode.data]=layoutNode
            for i in range(self.GetModel().GetChildCount(layoutNode.data)):
                p = MVCTreeNode("RAW", layoutNode, [])
                layoutNode.Add(p)
                p.data = self.GetModel().GetChildAt(layoutNode.data, i)
                self.nodemap[p.data]=p
            layoutNode.built = True
            if not self._assumeChildren:
                for kid in layoutNode.kids:
                    self.LoadChildren(kid)

    def OnEraseBackground(self, evt):
        pass

    def OnSize(self, evt):
        size = self.GetSize()
        self.center = (size.width/2, size.height/2)
        if self._lastPhysicalSize.width < size.width or self._lastPhysicalSize.height < size.height:
            self.painter.ClearBuffer()
        self._lastPhysicalSize = size

    def GetSelection(self):
        "Returns a tuple of selected nodes."
        return tuple(self._selections)

    def SetSelection(self, nodeTuple):
        if type(nodeTuple) != type(()):
            nodeTuple = (nodeTuple,)
        e = wxMVCTreeNotifyEvent(wxEVT_MVCTREE_SEL_CHANGING, self.GetId(), nodeTuple[0], nodes = nodeTuple)
        self.GetEventHandler().ProcessEvent(e)
        if not e.notify.IsAllowed():
            return
        for node in nodeTuple:
            treenode = self.nodemap[node]
            treenode.selected = True
        for node in self._selections:
            treenode = self.nodemap[node]
            node.selected = False
        self._selections = list(nodeTuple)
        e = wxMVCTreeEvent(wxEVT_MVCTREE_SEL_CHANGED, self.GetId(), nodeTuple[0], nodes = nodeTuple)
        self.GetEventHandler().ProcessEvent(e)

    def IsMultiSelect(self):
        return self._multiselect

    def SetMultiSelect(self, bool):
        self._multiselect = bool

    def IsSelected(self, node):
        return self.nodemap[node].selected

    def Edit(self, node):
        if not self.model.IsEditable(node):
            return
        for ed in self._editors:
            if ed.CanEdit(node):
                e = wxMVCTreeNotifyEvent(wxEVT_MVCTREE_BEGIN_EDIT, self.GetId(), node)
                self.GetEventHandler().ProcessEvent(e)
                if not e.notify.IsAllowed():
                    return
                ed.Edit(node)
                self._currentEditor = ed
                break

    def EndEdit(self):
        if self._currentEditor:
            self._currentEditor.EndEdit
            self._currentEditor = None

    def _EditEnding(self, node):
        e = wxMVCTreeNotifyEvent(wxEVT_MVCTREE_END_EDIT, self.GetId(), node)
        self.GetEventHandler().ProcessEvent(e)
        if not e.notify.IsAllowed():
            return False
        self._currentEditor = None
        return True


    def SetExpanded(self, node, bool):
        treenode = self.nodemap[node]
        if bool:
            e = wxMVCTreeNotifyEvent(wxEVT_MVCTREE_ITEM_EXPANDING, self.GetId(), node)
            self.GetEventHandler().ProcessEvent(e)
            if not e.notify.IsAllowed():
                return
            if not treenode.built:
                self.LoadChildren(treenode)
        else:
            e = wxMVCTreeNotifyEvent(wxEVT_MVCTREE_ITEM_COLLAPSING, self.GetId(), node)
            self.GetEventHandler().ProcessEvent(e)
            if not e.notify.IsAllowed():
                return
        treenode.expanded = bool
        e = None
        if treenode.expanded:
            e = wxMVCTreeEvent(wxEVT_MVCTREE_ITEM_EXPANDED, self.GetId(), node)
        else:
            e = wxMVCTreeEvent(wxEVT_MVCTREE_ITEM_COLLAPSED, self.GetId(), node)
        self.GetEventHandler().ProcessEvent(e)
        self.layout.Layout(self.currentRoot)
        self.transform.Transform(self.currentRoot, self.offset, self.rotation)
        self.Refresh()

    def IsExpanded(self, node):
        return self.nodemap[node].expanded

    def AddToSelection(self, nodeOrTuple, enableMulti = True, shiftMulti = False):
        nodeTuple = nodeOrTuple
        if type(nodeOrTuple)!= type(()):
            nodeTuple = (nodeOrTuple,)
        e = wxMVCTreeNotifyEvent(wxEVT_MVCTREE_SEL_CHANGING, self.GetId(), nodeTuple[0], nodes = nodeTuple)
        self.GetEventHandler().ProcessEvent(e)
        if not e.notify.IsAllowed():
            return
        changeparents = []
        if not (self.IsMultiSelect() and (enableMulti or shiftMulti)):
            for node in self._selections:
                treenode = self.nodemap[node]
                treenode.selected = False
                changeparents.append(treenode)
            node = nodeTuple[0]
            self._selections = [node]
            treenode = self.nodemap[node]
            changeparents.append(treenode)
            treenode.selected = True
        else:
            if shiftMulti:
                for node in nodeTuple:
                    treenode = self.nodemap[node]
                    oldtreenode = self.nodemap[self._selections[0]]
                    if treenode.parent == oldtreenode.parent:
                        found = 0
                        for kid in oldtreenode.parent.kids:
                            if kid == treenode or kid == oldtreenode:
                                found = not found
                                kid.selected = True
                                self._selections.append(kid.data)
                                changeparents.append(kid)
                            elif found:
                                kid.selected = True
                                self._selections.append(kid.data)
                                changeparents.append(kid)
            else:
                for node in nodeTuple:
                    try:
                        self._selections.index(node)
                    except ValueError:
                        self._selections.append(node)
                        treenode = self.nodemap[node]
                        treenode.selected = True
                        changeparents.append(treenode)
        e = wxMVCTreeEvent(wxEVT_MVCTREE_SEL_CHANGED, self.GetId(), nodeTuple[0], nodes = nodeTuple)
        self.GetEventHandler().ProcessEvent(e)
        dc = wxClientDC(self)
        self.PrepareDC(dc)
        for node in changeparents:
            if node:
                self.painter.Paint(dc, node, doubleBuffered = 0, paintBackground = 0)
        self.painter.ClearBuffer()

    def RemoveFromSelection(self, nodeTuple):
        if type(nodeTuple) != type(()):
            nodeTuple = (nodeTuple,)
        changeparents = []
        for node in nodeTuple:
            self._selections.remove(node)
            treenode = self.nodemap[node]
            changeparents.append(treenode)
            treenode.selected = False
        e = wxMVCTreeEvent(wxEVT_MVCTREE_SEL_CHANGED, self.GetId(), node, nodes = nodeTuple)
        self.GetEventHandler().ProcessEvent(e)
        dc = wxClientDC(self)
        self.PrepareDC(dc)
        for node in changeparents:
            if node:
                self.painter.Paint(dc, node, doubleBuffered = 0, paintBackground = 0)
        self.painter.ClearBuffer()


    def GetBackgroundColour(self):
        if hasattr(self, 'painter') and self.painter:
            return self.painter.GetBackgroundColour()
        else:
            return wxWindow.GetBackgroundColour(self)
    def SetBackgroundColour(self, color):
        if hasattr(self, 'painter') and self.painter:
            self.painter.SetBackgroundColour(color)
        else:
            wxWindow.SetBackgroundColour(self, color)
    def GetForegroundColour(self):
        if hasattr(self, 'painter') and self.painter:
            return self.painter.GetForegroundColour()
        else:
            return wxWindow.GetBackgroundColour(self)
    def SetForegroundColour(self, color):
        if hasattr(self, 'painter') and self.painter:
            self.painter.SetForegroundColour(color)
        else:
            wxWindow.SetBackgroundColour(self, color)

    def SetAssumeChildren(self, bool):
        self._assumeChildren = bool

    def GetAssumeChildren(self):
        return self._assumeChildren

    def OnPaint(self, evt):
        """
        Ensures that the tree has been laid out and transformed, then calls the painter
        to paint the control.
        """
        try:
            self.EnableScrolling(False, False)
            if not self.laidOut:
                self.layout.Layout(self.currentRoot)
                self.laidOut = True
                self.transformed = False
            if not self.transformed:
                self.transform.Transform(self.currentRoot, self.offset, self.rotation)
                self.transformed = True
            tsize = None
            tsize = list(self.transform.GetSize())
            tsize[0] = tsize[0] + 50
            tsize[1] = tsize[1] + 50
            size = self.GetSizeTuple()
            if tsize[0] > size[0] or tsize[1] > size[1]:
                if not hasattr(self, '_oldsize') or (tsize[0] > self._oldsize[0] or tsize[1] > self._oldsize[1]):
                    self._oldsize = tsize
                    oldstart = self.ViewStart()
                    self._lastPhysicalSize = self.GetSize()
                    self.SetScrollbars(10, 10, tsize[0]/10, tsize[1]/10)
                    self.Scroll(oldstart[0], oldstart[1])
            dc = wxPaintDC(self)
            self.PrepareDC(dc)
            dc.SetFont(self.GetFont())
            self.painter.Paint(dc, self.currentRoot, self.doubleBuffered)
        except:
            traceback.print_exc()















# Name:         globals.py
# Purpose:      XRC editor, global variables
# Author:       Roman Rolinsky <rolinsky@mema.ucl.ac.be>
# Created:      02.12.2002
# RCS-ID:       $Id: globals.py,v 1.1.2.8 2003/03/21 00:25:28 ROL Exp $

from wxPython.wx import *
from wxPython.xrc import *

# Global constants

sysFont = wxSystemSettings_GetFont(wxSYS_SYSTEM_FONT)
labelFont = wxFont(sysFont.GetPointSize(), wxDEFAULT, wxNORMAL, wxBOLD)
modernFont = wxFont(sysFont.GetPointSize(), wxMODERN, wxNORMAL, wxNORMAL)
smallerFont = wxFont(sysFont.GetPointSize()-2, wxDEFAULT, wxNORMAL, wxNORMAL)

progname = 'XRCed'
version = '0.1.0'

try:
    True
except NameError:
    True = 1==1
    False = 1==0

# Global variables

class Globals:
    panel = None
    tree = None
    frame = None
    tools = None
    undoMan = None
    testWin = None
    testWinPos = wxDefaultPosition
    currentXXX = None
    xmlFlags = wxXRC_USE_LOCALE | wxXRC_NO_SUBCLASSING

g = Globals()



"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: Joystick.py,v 1.1.2.1 2003/03/11 19:37:12 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Base import Object
import Parameters as wx


class Joystick(Object):
    """"""

    def GetButtonState(self):
        """"""
        pass

    def GetManufacturerId(self):
        """"""
        pass

    def GetMaxAxes(self):
        """"""
        pass

    def GetMaxButtons(self):
        """"""
        pass

    def GetMovementThreshold(self):
        """"""
        pass

    def GetNumberAxes(self):
        """"""
        pass

    def GetNumberButtons(self):
        """"""
        pass

    def GetNumberJoysticks(self):
        """"""
        pass

    def GetPOVCTSPosition(self):
        """"""
        pass

    def GetPOVPosition(self):
        """"""
        pass

    def GetPollingMax(self):
        """"""
        pass

    def GetPollingMin(self):
        """"""
        pass

    def GetPosition(self):
        """"""
        pass

    def GetProductId(self):
        """"""
        pass

    def GetProductName(self):
        """"""
        pass

    def GetRudderMax(self):
        """"""
        pass

    def GetRudderMin(self):
        """"""
        pass

    def GetRudderPosition(self):
        """"""
        pass

    def GetUMax(self):
        """"""
        pass

    def GetUMin(self):
        """"""
        pass

    def GetUPosition(self):
        """"""
        pass

    def GetVMax(self):
        """"""
        pass

    def GetVMin(self):
        """"""
        pass

    def GetVPosition(self):
        """"""
        pass

    def GetXMax(self):
        """"""
        pass

    def GetXMin(self):
        """"""
        pass

    def GetYMax(self):
        """"""
        pass

    def GetYMin(self):
        """"""
        pass

    def GetZMax(self):
        """"""
        pass

    def GetZMin(self):
        """"""
        pass

    def GetZPosition(self):
        """"""
        pass

    def HasPOV(self):
        """"""
        pass

    def HasPOV4Dir(self):
        """"""
        pass

    def HasPOVCTS(self):
        """"""
        pass

    def HasRudder(self):
        """"""
        pass

    def HasU(self):
        """"""
        pass

    def HasV(self):
        """"""
        pass

    def HasZ(self):
        """"""
        pass

    def IsOk(self):
        """"""
        pass

    def ReleaseCapture(self):
        """"""
        pass

    def SetCapture(self):
        """"""
        pass

    def SetMovementThreshold(self):
        """"""
        pass

    def __del__(self):
        """"""
        pass

    def __init__(self):
        """"""
        pass




# Name:         CDate.py
# Purpose:      Date and Calendar classes
#
# Author:       Lorne White (email: lwhite1@planet.eon.net)
#
# Created:
# Version       0.2 1999/11/08
# Licence:      wxWindows license
#----------------------------------------------------------------------------

import time

Month = {2: 'February', 3: 'March', None: 0, 'July': 7, 11:
         'November', 'December': 12, 'June': 6, 'January': 1, 'September': 9,
         'August': 8, 'March': 3, 'November': 11, 'April': 4, 12: 'December',
         'May': 5, 10: 'October', 9: 'September', 8: 'August', 7: 'July', 6:
         'June', 5: 'May', 4: 'April', 'October': 10, 'February': 2, 1:
         'January', 0: None}

# Number of days per month (except for February in leap years)
mdays = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]

# Full and abbreviated names of weekdays
day_name = [ 'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday']
day_abbr = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', ]

# Return number of leap years in range [y1, y2)
# Assume y1 <= y2 and no funny (non-leap century) years

def leapdays(y1, y2):
    return (y2+3)/4 - (y1+3)/4

# Return 1 for leap years, 0 for non-leap years
def isleap(year):
    return year % 4 == 0 and (year % 100 <> 0 or year % 400 == 0)

def FillDate(val):
    s = str(val)
    if len(s) < 2:
        s = '0' + s
    return s


def julianDay(year, month, day):
    b = 0L
    year, month, day = long(year), long(month), long(day)
    if month > 12L:
        year = year + month/12L
        month = month%12
    elif month < 1L:
        month = -month
        year = year - month/12L - 1L
        month = 12L - month%12L
    if year > 0L:
        yearCorr = 0L
    else:
        yearCorr = 3L
    if month < 3L:
        year = year - 1L
        month = month + 12L
    if year*10000L + month*100L + day > 15821014L:
        b = 2L - year/100L + year/400L
    return (1461L*year - yearCorr)/4L + 306001L*(month + 1L)/10000L + day + 1720994L + b


def TodayDay():
    date = time.localtime(time.time())
    year = date[0]
    month = date[1]
    day = date[2]
    julian = julianDay(year, month, day)
    daywk = dayOfWeek(julian)
    daywk = day_name[daywk]
    return(daywk)

def FormatDay(value):
    date = FromFormat(value)
    daywk = DateCalc.dayOfWeek(date)
    daywk = day_name[daywk]
    return(daywk)

def FromJulian(julian):
    julian = long(julian)
    if (julian < 2299160L):
        b = julian + 1525L
    else:
        alpha = (4L*julian - 7468861L)/146097L
        b = julian + 1526L + alpha - alpha/4L
    c = (20L*b - 2442L)/7305L
    d = 1461L*c/4L
    e = 10000L*(b - d)/306001L
    day = int(b - d - 306001L*e/10000L)
    if e < 14L:
        month = int(e - 1L)
    else:
        month = int(e - 13L)
    if month > 2:
        year = c - 4716L
    else:
        year = c - 4715L
    year = int(year)
    return year, month, day

def dayOfWeek(julian):
    return int((julian + 1L)%7L)

def daysPerMonth(month, year):
    ndays = mdays[month] + (month == 2 and isleap(year))
    return ndays

class now:
    def __init__(self):
        self.date = time.localtime(time.time())
        self.year = self.date[0]
        self.month = self.date[1]
        self.day = self.date[2]

class Date:
    def __init__(self, year, month, day):
        self.julian = julianDay(year, month, day)
        self.month = month
        self.year = year
        self.day_of_week = dayOfWeek(self.julian)
        self.days_in_month = daysPerMonth(self.month, self.year)




"""Decorator classes for documentation and shell scripting.
"""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: stc_.py,v 1.1.2.1 2003/03/11 19:37:11 RD Exp $"
__revision__ = "$Revision: 1.1.2.1 $"[11:-2]


# These are not the real wxPython classes. These are Python versions
# for documentation purposes. They are also used to apply docstrings
# to the real wxPython classes, which are SWIG-generated wrappers for
# C-language classes.


from Controls import Control
import Parameters as wx


class StyledTextCtrl(Control):
    """StyledTextCtrl class."""

    def __init__(self, parent, id, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0, name=wx.PySTCNameStr):
        """Create a StyledTextCtrl instance."""
        pass

    def AddRefDocument(self, docPointer):
        """Extend life of document."""
        pass

    def AddStyledText(self, data):
        """Add array of cells to document."""
        pass

    def AddText(self, text):
        """Add text to the document."""
        pass

    def AutoCompActive(self):
        """Is there an auto-completion list visible?"""
        pass

    def AutoCompCancel(self):
        """Remove the auto-completion list from the screen."""
        pass

    def AutoCompComplete(self):
        """User has selected an item so remove the list and insert the
        selection."""
        pass

    def AutoCompGetAutoHide(self):
        """Return whether or not autocompletion is hidden
        automatically when nothing matches."""
        pass

    def AutoCompGetCancelAtStart(self):
        """Return whether auto-completion cancelled by backspacing
        before start."""
        pass

    def AutoCompGetChooseSingle(self):
        """Return whether a single item auto-completion list
        automatically choose the item."""
        pass

    def AutoCompGetDropRestOfWord(self):
        """Return whether or not autocompletion deletes any word
        characters after the inserted text upon completion."""
        pass

    def AutoCompGetIgnoreCase(self):
        """Return state of ignore case flag."""
        pass

    def AutoCompGetSeparator(self):
        """Return the auto-completion list separator character."""
        pass

    def AutoCompPosStart(self):
        """Return the position of the caret when the auto-completion
        list was displayed."""
        pass

    def AutoCompSelect(self, text):
        """Select the item in the auto-completion list that starts
        with a string."""
        pass

    def AutoCompSetAutoHide(self, autoHide):
        """Set whether or not autocompletion is hidden automatically
        when nothing matches."""
        pass

    def AutoCompSetCancelAtStart(self, cancel):
        """Should the auto-completion list be cancelled if the user
        backspaces to a position before where the box was created."""
        pass

    def AutoCompSetChooseSingle(self, chooseSingle):
        """Should a single item auto-completion list automatically
        choose the item."""
        pass

    def AutoCompSetDropRestOfWord(self, dropRestOfWord):
        """Set whether or not autocompletion deletes any word
        characters after the inserted text upon completion."""
        pass

    def AutoCompSetFillUps(self, characterSet):
        """Define a set of characters that when typed will cause the
        autocompletion to choose the selected item."""
        pass

    def AutoCompSetIgnoreCase(self, ignoreCase):
        """Set whether case is significant when performing
        auto-completion searches."""
        pass

    def AutoCompSetSeparator(self, separatorCharacter):
        """Change the separator character in the string setting up an
        auto-completion list.  Default is space but can be changed if
        items contain space."""
        pass

    def AutoCompShow(self, lenEntered, itemList):
        """Display a auto-completion list.

        The lenEntered parameter indicates how many characters before
        the caret should be used to provide context."""
        pass

    def AutoCompStops(self, characterSet):
        """Define a set of character that when typed cancel the
        auto-completion list."""
        pass

    def BeginUndoAction(self):
        """Start a sequence of actions that is undone and redone as a
        unit.  May be nested."""
        pass

    def BraceBadLight(self, pos):
        """Highlight the character at a position indicating there is
        no matching brace."""
        pass

    def BraceHighlight(self, pos1, pos2):
        """Highlight the characters at two positions."""
        pass

    def BraceMatch(self, pos):
        """Find the position of a matching brace or INVALID_POSITION
        if no match."""
        pass

    def CallTipActive(self):
        """Is there an active call tip?"""
        pass

    def CallTipCancel(self):
        """Remove the call tip from the screen."""
        pass

    def CallTipPosAtStart(self):
        """Return the position where the caret was before displaying
        the call tip."""
        pass

    def CallTipSetBackground(self, back):
        """Set the background colour for the call tip."""
        pass

    def CallTipSetHighlight(self, start, end):
        """Highlight a segment of the definition."""
        pass

    def CallTipShow(self, pos, definition):
        """Show a call tip containing a definition near position pos."""
        pass

    def CanPaste(self):
        """Will a paste succeed?"""
        pass

    def CanRedo(self):
        """Are there any redoable actions in the undo history?"""
        pass

    def CanUndo(self):
        """Are there any undoable actions in the undo history?"""
        pass

    def Clear(self):
        """Clear the selection."""
        pass

    def ClearAll(self):
        """Delete all text in the document."""
        pass

    def ClearDocumentStyle(self):
        """Set all style bytes to 0, remove all folding information."""
        pass

    def CmdKeyAssign(self, key, modifiers, cmd):
        """When key+modifier combination km is pressed perform msg."""
        pass

    def CmdKeyClear(self, key, modifiers):
        """When key+modifier combination km do nothing."""
        pass

    def CmdKeyClearAll(self):
        """Drop all key mappings."""
        pass

    def CmdKeyExecute(self, cmd):
        """Perform one of the operations defined by the STC_CMD_*
        constants."""
        pass

    def Colourise(self, start, end):
        """Colourise a segment of the document using the current
        lexing language."""
        pass

    def ConvertEOLs(self, eolMode):
        """Convert all line endings in the document to one mode."""
        pass

    def Copy(self):
        """Copy the selection to the clipboard."""
        pass

    def CreateDocument(self):
        """Create a new document object.

        Starts with reference count of 1 and not selected into editor."""
        pass

    def Cut(self):
        """Cut the selection to the clipboard."""
        pass

    def DelLineLeft(self):
        """Delete from the current position to the start of the line."""
        pass

    def DelLineRight(self):
        """Delete from the current position to the end of the line."""
        pass

    def DocLineFromVisible(self, lineDisplay):
        """Find the document line of a display line taking hidden
        lines into account."""
        pass

    def EmptyUndoBuffer(self):
        """Delete the undo history."""
        pass

    def EndUndoAction(self):
        """End a sequence of actions that is undone and redone as a
        unit."""
        pass

    def EnsureCaretVisible(self):
        """Ensure the caret is visible."""
        pass

    def EnsureVisible(self, line):
        """Ensure a particular line is visible by expanding any header
        line hiding it."""
        pass

    def EnsureVisibleEnforcePolicy(self, line):
        """Ensure a particular line is visible by expanding any header
        line hiding it.  Use the currently set visibility policy to
        determine which range to display."""
        pass

    def FindText(self, minPos, maxPos, text, flags=0):
        """Find some text in the document."""
        pass

    def FormatRange(self, doDraw, startPos, endPos, draw,
                    target, renderRect, pageRect):
        """On Windows, will draw the document into a display context
        such as a printer."""
        pass

    def GetAnchor(self):
        """Return the position of the opposite end of the selection to
        the caret."""
        pass

    def GetBackSpaceUnIndents(self):
        """Does a backspace pressed when caret is within indentation
        unindent?"""
        pass

    def GetBufferedDraw(self):
        """Is drawing done first into a buffer or direct to the
        screen?"""
        pass

    def GetCaretForeground(self):
        """Return the foreground colour of the caret."""
        pass

    def GetCaretLineBack(self):
        """Return the colour of the background of the line containing
        the caret."""
        pass

    def GetCaretLineVisible(self):
        """Is the background of the line containing the caret in a
        different colour?"""
        pass

    def GetCaretPeriod(self):
        """Return the time in milliseconds that the caret is on and
        off."""
        pass

    def GetCaretWidth(self):
        """Return the width of the insert mode caret."""
        pass

    def GetCharAt(self, pos):
        """Return the character byte at the position."""
        pass

    def GetCodePage(self):
        """Return the code page used to interpret the bytes of the
        document as characters."""
        pass

    def GetColumn(self, pos):
        """Return the column number of a position, taking tab width
        into account."""
        pass

    def GetControlCharSymbol(self):
        """Return the way control characters are displayed."""
        pass

    def GetCurLine(self):
        """Return (text, pos) tuple with the text of the line
        containing the caret and the position of the caret within the
        line."""
        pass

    def GetCurrentLine(self):
        """Return the line number of the line with the caret."""
        pass

    def GetCurrentPos(self):
        """Return the position of the caret."""
        pass

    def GetCursor(self):
        """Return cursor type."""
        pass

    def GetDocPointer(self):
        """Return a pointer to the document object."""
        pass

    def GetEOLMode(self):
        """Return the current end of line mode: CRLF, CR, or LF."""
        pass

    def GetEdgeColour(self):
        """Return the colour used in edge indication."""
        pass

    def GetEdgeColumn(self):
        """Return the column number which text should be kept within."""
        pass

    def GetEdgeMode(self):
        """Return the edge highlight mode."""
        pass

    def GetEndAtLastLine(self):
        """Return whether the maximum scroll position has the last
        line at the bottom of the view."""
        pass

    def GetEndStyled(self):
        """Return the position of the last correctly styled character."""
        pass

    def GetFirstVisibleLine(self):
        """Return the line at the top of the display."""
        pass

    def GetFoldExpanded(self, line):
        """Is a header line expanded?"""
        pass

    def GetFoldLevel(self, line):
        """Return the fold level of a line."""
        pass

    def GetFoldParent(self, line):
        """Find the parent line of a child line."""
        pass

    def GetHighlightGuide(self):
        """Return the highlighted indentation guide column."""
        pass

    def GetIndent(self):
        """Return indentation size."""
        pass

    def GetIndentationGuides(self):
        """Are the indentation guides visible?"""
        pass

    def GetLastChild(self, line, level):
        """Find the last child line of a header line."""
        pass

    def GetLastKeydownProcessed(self):
        """Can be used to prevent the EVT_CHAR handler from adding the
        char."""
        pass

    def GetLayoutCache(self):
        """Return the degree of caching of layout information."""
        pass

    def GetLength(self):
        """Return the number of characters in the document."""
        pass

    def GetLexer(self):
        """Return the lexing language of the document."""
        pass

    def GetLine(self, line):
        """Return the contents of a line."""
        pass

    def GetLineCount(self):
        """Return the number of lines in the document. There is
        always at least one."""
        pass

    def GetLineEndPosition(self, line):
        """Return the position after the last visible characters on a
        line."""
        pass

    def GetLineIndentPosition(self, line):
        """Return the position before the first non indentation
        character on a line."""
        pass

    def GetLineIndentation(self, line):
        """Return the number of columns that a line is indented."""
        pass

    def GetLineState(self, line):
        """Return the extra styling information for a line."""
        pass

    def GetLineVisible(self, line):
        """Is a line visible?"""
        pass

    def GetMarginLeft(self):
        """Return the size in pixels of the left margin."""
        pass

    def GetMarginMask(self, margin):
        """Return the marker mask of a margin."""
        pass

    def GetMarginRight(self):
        """Return the size in pixels of the right margin."""
        pass

    def GetMarginSensitive(self, margin):
        """Return the mouse click sensitivity of a margin."""
        pass

    def GetMarginType(self, margin):
        """Return the type of a margin."""
        pass

    def GetMarginWidth(self, margin):
        """Return the width of a margin in pixels."""
        pass

    def GetMaxLineState(self):
        """Return the last line number that has line state."""
        pass

    def GetModEventMask(self):
        """Return which document modification events are sent to the
        container."""
        pass

    def GetModify(self):
        """Is the document different from when it was last saved?"""
        pass

    def GetMouseDownCaptures(self):
        """Return whether mouse gets captured."""
        pass

    def GetMouseDwellTime(self):
        """Return the time the mouse must sit still to generate a
        mouse dwell event."""
        pass

    def GetOvertype(self):
        """Return true if overtype mode is active otherwise false is
        returned."""
        pass

    def GetPrintColourMode(self):
        """Return the print colour mode."""
        pass

    def GetPrintMagnification(self):
        """Return the print magnification."""
        pass

    def GetReadOnly(self):
        """In read-only mode?"""
        pass

    def GetSTCFocus(self):
        """Return internal focus flag."""
        pass

    def GetScrollWidth(self):
        """Return the document width assumed for scrolling."""
        pass

    def GetSearchFlags(self):
        """Return the search flags used by SearchInTarget."""
        pass

    def GetSelectedText(self):
        """Return the selected text."""
        pass

    def GetSelectionEnd(self):
        """Return the position at the end of the selection."""
        pass

    def GetSelection(self):
        """Return the start and end positions of the current
        selection."""
        pass

    def GetSelectionStart(self):
        """Return the position at the start of the selection."""
        pass

    def GetStatus(self):
        """Return error status."""
        pass

    def GetStyleAt(self, pos):
        """Return the style byte at the position."""
        pass

    def GetStyleBits(self):
        """Return number of bits in style bytes used to hold the
        lexical state."""
        pass

    def GetStyledText(self, startPos, endPos):
        """Return a buffer of cells."""
        pass

    def GetTabIndents(self):
        """Does a tab pressed when caret is within indentation indent?"""
        pass

    def GetTabWidth(self):
        """Return the visible size of a tab."""
        pass

    def GetTargetEnd(self):
        """Return the position that ends the target."""
        pass

    def GetTargetStart(self):
        """Return the position that starts the target."""
        pass

    def GetText(self):
        """Return all the text in the document."""
        pass

    def GetTextLength(self):
        """Return the number of characters in the document."""
        pass

    def GetTextRange(self, startPos, endPos):
        """Return a range of text."""
        pass

    def GetUndoCollection(self):
        """Is undo history being collected?"""
        pass

    def GetUseHorizontalScrollBar(self):
        """Is the horizontal scroll bar visible?"""
        pass

    def GetUseTabs(self):
        """Return whether tabs will be used in indentation."""
        pass

    def GetViewEOL(self):
        """Are the end of line characters visible?"""
        pass

    def GetViewWhiteSpace(self):
        """Are white space characters currently visible? Return one
        of SCWS_* constants."""
        pass

    def GetWrapMode(self):
        """Return whether text is word wrapped."""
        pass

    def GetXOffset(self):
        """Return the xOffset (self, ie, horizonal scroll position)."""
        pass

    def GetZoom(self):
        """Return the zoom level."""
        pass

    def GotoLine(self, line):
        """Set caret to start of a line and ensure it is visible."""
        pass

    def GotoPos(self, pos):
        """Set caret to a position and ensure it is visible."""
        pass

    def HideLines(self, lineStart, lineEnd):
        """Make a range of lines invisible."""
        pass

    def HideSelection(self, normal):
        """Draw the selection in normal style or with selection
        highlighted."""
        pass

    def HomeDisplay(self):
        """Move caret to first position on display line."""
        pass

    def HomeDisplayExtend(self):
        """Move caret to first position on display line extending
        selection to new caret position."""
        pass

    def IndicatorGetForeground(self, indic):
        """Return the foreground colour of an indicator."""
        pass

    def IndicatorGetStyle(self, indic):
        """Return the style of an indicator."""
        pass

    def IndicatorSetForeground(self, indic, fore):
        """Set the foreground colour of an indicator."""
        pass

    def IndicatorSetStyle(self, indic, style):
        """Set an indicator to plain, squiggle or TT."""
        pass

    def InsertText(self, pos, text):
        """Insert string at a position."""
        pass

    def LineEndDisplay(self):
        """Move caret to last position on display line."""
        pass

    def LineEndDisplayExtend(self):
        """Move caret to last position on display line extending
        selection to new caret position."""
        pass

    def LineFromPosition(self, pos):
        """Return the line containing a position."""
        pass

    def LineLength(self, line):
        """How many characters are on a line, not including end of
        line characters?"""
        pass

    def LineScroll(self, columns, lines):
        """Scroll horizontally and vertically."""
        pass

    def LinesOnScreen(self):
        """Retrieves the number of lines completely visible."""
        pass

    def MarkerAdd(self, line, markerNumber):
        """Add a marker to a line, return an ID that can be used to
        find or delete the marker."""
        pass

    def MarkerDefine(self, markerNumber, markerSymbol,
                     foreground=wx.NullColour, background=wx.NullColour):
        """Set the symbol used for a particular marker number, and
        optionally the fore and background colours."""
        pass

    def MarkerDelete(self, line, markerNumber):
        """Delete a marker from a line."""
        pass

    def MarkerDeleteAll(self, markerNumber):
        """Delete all markers with a particular number from all lines."""
        pass

    def MarkerDeleteHandle(self, handle):
        """Delete a marker."""
        pass

    def MarkerGet(self, line):
        """Return a bit mask of all the markers set on a line."""
        pass

    def MarkerLineFromHandle(self, handle):
        """Return the line number at which a particular marker is
        located."""
        pass

    def MarkerNext(self, lineStart, markerMask):
        """Find the next line after lineStart that includes a marker
        in mask."""
        pass

    def MarkerPrevious(self, lineStart, markerMask):
        """Find the previous line before lineStart that includes a
        marker in mask."""
        pass

    def MarkerSetBackground(self, markerNumber, back):
        """Set the background colour used for a particular marker
        number."""
        pass

    def MarkerSetForeground(self, markerNumber, fore):
        """Set the foreground colour used for a particular marker
        number."""
        pass

    def MoveCaretInsideView(self):
        """Move the caret inside current view if it is not there
        already."""
        pass

    def Paste(self):
        """Paste the contents of the clipboard into the document
        replacing the selection."""
        pass

    def PointFromPosition(self, pos):
        """Return the point in the window where a position is
        displayed."""
        pass

    def PositionFromLine(self, line):
        """Return the position at the start of a line."""
        pass

    def PositionFromPoint(self, pt):
        """Find the position from a point within the window."""
        pass

    def PositionFromPointClose(self, x, y):
        """Find the position from a point within the window but return
        INVALID_POSITION if not close to text."""
        pass

    def Redo(self):
        """Redo the next action on the undo history."""
        pass

    def ReleaseDocument(self, docPointer):
        """Release a reference to the document, deleting document if
        it fades to black."""
        pass

    def ReplaceSelection(self, text):
        """Replace the selected text with the argument text."""
        pass

    def ReplaceTarget(self, text):
        """Replace the target text with the argument text.

        Text is counted so it can contain nulls.  Return the length
        of the replacement text."""
        pass

    def ReplaceTargetRE(self, text):
        """Replace the target text with the argument text after \d
        processing.

        Text is counted so it can contain nulls.  Looks for \d where d
        is between 1 and 9 and replaces these with the strings matched
        in the last search operation which were surrounded by \(self,
        and \).  Return the length of the replacement text including
        any change caused by processing the \d patterns."""
        pass

    def ScrollToColumn(self, column):
        """Scroll enough to make the given column visible."""
        pass

    def ScrollToLine(self, line):
        """Scroll enough to make the given line visible."""
        pass

    def SearchAnchor(self):
        """Sets the current caret position to be the search anchor."""
        pass

    def SearchInTarget(self, text):
        """Search for a counted string in the target and set the
        target to the found range.

        Text is counted so it can contain nulls.  Return length of
        range or -1 for failure in which case target is not moved."""
        pass

    def SearchNext(self, flags, text):
        """Find some text starting at the search anchor.  Does not
        ensure the selection is visible."""
        pass

    def SearchPrev(self, flags, text):
        """Find some text starting at the search anchor and moving
        backwards.  Does not ensure the selection is visible."""
        pass

    def SelectAll(self):
        """Select all the text in the document."""
        pass

    def SelectionIsRectangle(self):
        """Is the selection rectangular?  The alternative is the more
        common stream selection."""
        pass

    def SendMsg(self, msg, wp=0, lp=0):
        """Send a message to Scintilla."""
        pass

    def SetAnchor(self, posAnchor):
        """Set the selection anchor to a position.  The anchor is the
        opposite end of the selection from the caret."""
        pass

    def SetBackSpaceUnIndents(self, bsUnIndents):
        """Sets whether a backspace pressed when caret is within
        indentation unindents."""
        pass

    def SetBufferedDraw(self, buffered):
        """If drawing is buffered then each line of text is drawn into
        a bitmap buffer before drawing it to the screen to avoid
        flicker."""
        pass

    def SetCaretForeground(self, fore):
        """Set the foreground colour of the caret."""
        pass

    def SetCaretLineBack(self, back):
        """Set the colour of the background of the line containing the
        caret."""
        pass

    def SetCaretLineVisible(self, show):
        """Display the background of the line containing the caret in
        a different colour."""
        pass

    def SetCaretPeriod(self, periodMilliseconds):
        """Return the time in milliseconds that the caret is on and
        off. 0 = steady on."""
        pass

    def SetCaretWidth(self, pixelWidth):
        """Set the width of the insert mode caret."""
        pass

    def SetCodePage(self, codePage):
        """Set the code page used to interpret the bytes of the
        document as characters."""
        pass

    def SetControlCharSymbol(self, symbol):
        """Change the way control characters are displayed: If symbol
        is < 32, keep the drawn way, else, use the given character."""
        pass

    def SetCurrentPos(self, pos):
        """Sets the position of the caret."""
        pass

    def SetCursor(self, cursorType):
        """Sets the cursor to one of the SC_CURSOR* values."""
        pass

    def SetDocPointer(self, docPointer):
        """Change the document object used."""
        pass

    def SetEOLMode(self, eolMode):
        """Set the current end of line mode."""
        pass

    def SetEdgeColour(self, edgeColour):
        """Change the colour used in edge indication."""
        pass

    def SetEdgeColumn(self, column):
        """Set the column number of the edge.  If text goes past the
        edge then it is highlighted."""
        pass

    def SetEdgeMode(self, mode):
        """The edge may be displayed by a line (self, EDGE_LINE) or by
        highlighting text that goes beyond it (self, EDGE_BACKGROUND)
        or not displayed at all (self, EDGE_NONE)."""
        pass

    def SetEndAtLastLine(self, endAtLastLine):
        """Sets the scroll range so that maximum scroll position has
        the last line at the bottom of the view (self, default).
        Setting this to false allows scrolling one page below the last
        line."""
        pass

    def SetFoldExpanded(self, line, expanded):
        """Show the children of a header line."""
        pass

    def SetFoldFlags(self, flags):
        """Set some debugging options for folding."""
        pass

    def SetFoldLevel(self, line, level):
        """Set the fold level of a line.  This encodes an integer
        level along with flags indicating whether the line is a header
        and whether it is effectively white space."""
        pass

    def SetHScrollBar(self, bar):
        """Set the horizontal scrollbar to use instead of the one
        that's built-in."""
        pass

    def SetHighlightGuide(self, column):
        """Set the highlighted indentation guide column.  0 = no
        highlighted guide."""
        pass

    def SetIndent(self, indentSize):
        """Set the number of spaces used for one level of indentation."""
        pass

    def SetIndentationGuides(self, show):
        """Show or hide indentation guides."""
        pass

    def SetKeyWords(self, keywordSet, keyWords):
        """Set up the key words used by the lexer."""
        pass

    def SetLastKeydownProcessed(self, val):
        """Set ???."""
        pass

    def SetLayoutCache(self, mode):
        """Sets the degree of caching of layout information."""
        pass

    def SetLexer(self, lexer):
        """Set the lexing language of the document."""
        pass

    def SetLexerLanguage(self, language):
        """Set the lexing language of the document based on string
        name."""
        pass

    def SetLineIndentation(self, line, indentSize):
        """Change the indentation of a line to a number of columns."""
        pass

    def SetLineState(self, line, state):
        """Used to hold extra styling information for each line."""
        pass

    def SetMarginLeft(self, pixelWidth):
        """Sets the size in pixels of the left margin."""
        pass

    def SetMarginMask(self, margin, mask):
        """Set a mask that determines which markers are displayed in a
        margin."""
        pass

    def SetMarginRight(self, pixelWidth):
        """Sets the size in pixels of the right margin."""
        pass

    def SetMarginSensitive(self, margin, sensitive):
        """Make a margin sensitive or insensitive to mouse clicks."""
        pass

    def SetMarginType(self, margin, marginType):
        """Set a margin to be either numeric or symbolic."""
        pass

    def SetMarginWidth(self, margin, pixelWidth):
        """Set the width of a margin to a width expressed in pixels."""
        pass

    def SetMargins(self, left, right):
        """Set the left and right margin in the edit area, measured in
        pixels."""
        pass

    def SetModEventMask(self, mask):
        """Set which document modification events are sent to the
        container."""
        pass

    def SetMouseDownCaptures(self, captures):
        """Set whether the mouse is captured when its button is
        pressed."""
        pass

    def SetMouseDwellTime(self, periodMilliseconds):
        """Sets the time the mouse must sit still to generate a mouse
        dwell event."""
        pass

    def SetOvertype(self, overtype):
        """Set to overtype (self, true) or insert mode."""
        pass

    def SetPrintColourMode(self, mode):
        """Modify colours when printing for clearer printed text."""
        pass

    def SetPrintMagnification(self, magnification):
        """Sets the print magnification added to the point size of
        each style for printing."""
        pass

    def SetProperty(self, key, value):
        """Set up a value that may be used by a lexer for some
        optional feature."""
        pass

    def SetReadOnly(self, readOnly):
        """Set to read only or read write."""
        pass

    def SetSTCFocus(self, focus):
        """Change internal focus flag."""
        pass

    def SetSavePoint(self):
        """Remember the current position in the undo history as the
        position at which the document was saved."""
        pass

    def SetScrollWidth(self, pixelWidth):
        """Sets the document width assumed for scrolling."""
        pass

    def SetSearchFlags(self, flags):
        """Set the search flags used by SearchInTarget."""
        pass

    def SetSelBackground(self, useSetting, back):
        """Set the background colour of the selection and whether to
        use this setting."""
        pass

    def SetSelForeground(self, useSetting, fore):
        """Set the foreground colour of the selection and whether to
        use this setting."""
        pass

    def SetSelection(self, start, end):
        """Select a range of text."""
        pass

    def SetSelectionEnd(self, pos):
        """Sets the position that ends the selection - this becomes
        the currentPosition."""
        pass

    def SetSelectionStart(self, pos):
        """Sets the position that starts the selection - this becomes
        the anchor."""
        pass

    def SetStatus(self, statusCode):
        """Change error status - 0 = OK."""
        pass

    def SetStyleBits(self, bits):
        """Divide each styling byte into lexical class bits (self,
        default: 5) and indicator bits (self, default: 3). If a lexer
        requires more than 32 lexical states, then this is used to
        expand the possible states."""
        pass

    def SetStyleBytes(self, length, styleBytes):
        """Set the styles for a segment of the document."""
        pass

    def SetStyling(self, length, style):
        """Change style from current styling position for length
        characters to a style and move the current styling position to
        after this newly styled segment."""
        pass

    def SetTabIndents(self, tabIndents):
        """Sets whether a tab pressed when caret is within indentation
        indents."""
        pass

    def SetTabWidth(self, tabWidth):
        """Change the visible size of a tab to be a multiple of the
        width of a space character."""
        pass

    def SetTargetEnd(self, pos):
        """Sets the position that ends the target which is used for
        updating the document without affecting the scroll position."""
        pass

    def SetTargetStart(self, pos):
        """Sets the position that starts the target which is used for
        updating the document without affecting the scroll position."""
        pass

    def SetText(self, text):
        """Replace the contents of the document with the argument
        text."""
        pass

    def SetUndoCollection(self, collectUndo):
        """Choose between collecting actions into the undo history and
        discarding them."""
        pass

    def SetUseHorizontalScrollBar(self, show):
        """Show or hide the horizontal scroll bar."""
        pass

    def SetUseTabs(self, useTabs):
        """Indentation will only use space characters if useTabs is
        false, otherwise it will use a combination of tabs and spaces."""
        pass

    def SetVScrollBar(self, bar):
        """Set the vertical scrollbar to use instead of the one that's
        built-in."""
        pass

    def SetViewEOL(self, visible):
        """Make the end of line characters visible or invisible."""
        pass

    def SetViewWhiteSpace(self, viewWS):
        """Make white space characters invisible, always visible or
        visible outside indentation."""
        pass

    def SetVisiblePolicy(self, visiblePolicy, visibleSlop):
        """Set the way the display area is determined when a
        particular line is to be moved to by Find, FindNext, GotoLine,
        etc."""
        pass

    def SetWhitespaceBackground(self, useSetting, back):
        """Set the background colour of all whitespace and whether to
        use this setting."""
        pass

    def SetWhitespaceForeground(self, useSetting, fore):
        """Set the foreground colour of all whitespace and whether to
        use this setting."""
        pass

    def SetWordChars(self, characters):
        """Set the set of characters making up words for when moving
        or selecting by word."""
        pass

    def SetWrapMode(self, mode):
        """Sets whether text is word wrapped."""
        pass

    def SetXCaretPolicy(self, caretPolicy, caretSlop):
        """Set the way the caret is kept visible when going sideway.
        The exclusion zone is given in pixels."""
        pass

    def SetXOffset(self, newOffset):
        """Set the xOffset (self, ie, horizonal scroll position)."""
        pass

    def SetYCaretPolicy(self, caretPolicy, caretSlop):
        """Set the way the line the caret is on is kept visible.  The
        exclusion zone is given in lines."""
        pass

    def SetZoom(self, zoom):
        """Set the zoom level. This number of points is added to the
        size of all fonts.  It may be positive to magnify or negative
        to reduce."""
        pass

    def ShowLines(self, lineStart, lineEnd):
        """Make a range of lines visible."""
        pass

    def StartRecord(self):
        """Start notifying the container of all key presses and
        commands."""
        pass

    def StartStyling(self, pos, mask):
        """Set the current styling position and mask.

        The styling mask can be used to protect some bits in each
        styling byte from modification."""
        pass

    def StopRecord(self):
        """Stop notifying the container of all key presses and
        commands."""
        pass

    def StyleClearAll(self):
        """Clear all the styles and make equivalent to the global
        default style."""
        pass

    def StyleResetDefault(self):
        """Reset the default style to its state at startup."""
        pass

    def StyleSetBackground(self, style, back):
        """Set the background colour of a style."""
        pass

    def StyleSetBold(self, style, bold):
        """Set a style to be bold or not."""
        pass

    def StyleSetCase(self, style, caseForce):
        """Set a style to be mixed case, or to force upper or lower
        case."""
        pass

    def StyleSetChangeable(self, style, changeable):
        """Set a style to be changeable or not (self, read only).
        Experimental feature, currently buggy."""
        pass

    def StyleSetCharacterSet(self, style, characterSet):
        """Set the character set of the font in a style."""
        pass

    def StyleSetEOLFilled(self, style, filled):
        """Set a style to have its end of line filled or not."""
        pass

    def StyleSetFaceName(self, style, fontName):
        """Set the font of a style."""
        pass

    def StyleSetFont(self, styleNum, font):
        """Set style size, face, bold, italic, and underline
        attributes from a Font's attributes."""
        pass

    def StyleSetFontAttr(self, styleNum, size, faceName,
                         bold, italic, underline):
        """Set all font style attributes at once."""
        pass

    def StyleSetForeground(self, style, fore):
        """Set the foreground colour of a style."""
        pass

    def StyleSetItalic(self, style, italic):
        """Set a style to be italic or not."""
        pass

    def StyleSetSize(self, style, sizePoints):
        """Set the size of characters of a style."""
        pass

    def StyleSetSpec(self, styleNum, spec):
        """Extract style settings from a spec-string which is composed
        of one or more of the following comma separated elements:

        bold             turns on bold
        italic           turns on italics
        fore:#RRGGBB     sets the foreground colour
        back:#RRGGBB     sets the background colour
        face:[facename]  sets the font face name to use
        size:[num]       sets the font size in points
        eol              turns on eol filling
        underline        turns on underlining
        """
        pass

    def StyleSetUnderline(self, style, underline):
        """Set a style to be underlined or not."""
        pass

    def StyleSetVisible(self, style, visible):
        """Set a style to be visible or not."""
        pass

    def TextHeight(self, line):
        """Return the height of a particular line of text in pixels."""
        pass

    def TextWidth(self, style, text):
        """Measure the pixel width of some text in a particular style.
        Nul terminated text argument.  Does not handle tab or control
        characters."""
        pass

    def ToggleFold(self, line):
        """Switch a header line between expanded and contracted."""
        pass

    def Undo(self):
        """Undo one action in the undo history."""
        pass

    def UsePopUp(self, allowPopUp):
        """Set whether a pop up menu is displayed automatically when
        the user presses the wrong mouse button."""
        pass

    def UserListShow(self, listType, itemList):
        """Display a list of strings and send notification when user
        chooses one."""
        pass

    def VisibleFromDocLine(self, line):
        """Find the display line of a document line taking hidden
        lines into account."""
        pass

    def WordEndPosition(self, pos, onlyWordCharacters):
        """Return position of end of word."""
        pass

    def WordPartLeft(self):
        """Move to the previous change in capitalisation."""
        pass

    def WordPartLeftExtend(self):
        """Move to the previous change in capitalisation extending
        selection to new caret position."""
        pass

    def WordPartRight(self):
        """Move to the change next in capitalisation."""
        pass

    def WordPartRightExtend(self):
        """Move to the next change in capitalisation extending
        selection to new caret position."""
        pass

    def WordStartPosition(self, pos, onlyWordCharacters):
        """Return position of start of word."""
        pass
